@@ -2,12 +2,12 @@
 #define defgMouseWinSize                XK_Pointer_Button3, kfAlt, "Alt+Pointer_Button3"
 #define defgMouseWinRaise               XK_Pointer_Button1, kfCtrl+kfAlt, "Ctrl+Alt+Pointer_Button1"
 #define defgMouseWinLower               XK_Pointer_Button1, kfCtrl+kfAlt, "Ctrl+Alt+Pointer_Button1"
+#define defgMouseWinTabbing             XK_Pointer_Button2, kfShift, "Shift+Pointer_Button2"
 
 #define defgKeyWinRaise                 XK_F1, kfAlt, "Alt+F1"
 #define defgKeyWinOccupyAll             XK_F2, kfAlt, "Alt+F2"
 #define defgKeyWinLower                 XK_F3, kfAlt, "Alt+F3"
 #define defgKeyWinClose                 XK_F4, kfAlt, "Alt+F4"
-#define defgKeyWinKill                  0, 0, ""
 #define defgKeyWinRestore               XK_F5, kfAlt, "Alt+F5"
 #define defgKeyWinNext                  XK_F6, kfAlt, "Alt+F6"
 #define defgKeyWinPrev                  XK_F6, kfAlt+kfShift, "Alt+Shift+F6"
@@ -20,24 +20,25 @@
 #define defgKeyWinFullscreen            XK_F11, kfAlt, "Alt+F11"
 #define defgKeyWinHide                  XK_F12, kfAlt+kfShift, "Alt+Shift+F12"
 #define defgKeyWinRollup                XK_F12, kfAlt, "Alt+F12"
-#define defgKeyWinArrangeN              XK_KP_Up, kfCtrl+kfAlt, "Ctrl+Alt+KP_8"
-#define defgKeyWinArrangeNE             XK_KP_Prior, kfCtrl+kfAlt, "Ctrl+Alt+KP_9"
-#define defgKeyWinArrangeE              XK_KP_Right, kfCtrl+kfAlt, "Ctrl+Alt+KP_6"
-#define defgKeyWinArrangeSE             XK_KP_Next, kfCtrl+kfAlt, "Ctrl+Alt+KP_3"
-#define defgKeyWinArrangeS              XK_KP_Down, kfCtrl+kfAlt, "Ctrl+Alt+KP_2"
-#define defgKeyWinArrangeSW             XK_KP_End, kfCtrl+kfAlt, "Ctrl+Alt+KP_1"
-#define defgKeyWinArrangeW              XK_KP_Left, kfCtrl+kfAlt, "Ctrl+Alt+KP_4"
-#define defgKeyWinArrangeNW             XK_KP_Home, kfCtrl+kfAlt, "Ctrl+Alt+KP_7"
-#define defgKeyWinArrangeC              XK_KP_Begin, kfCtrl+kfAlt, "Ctrl+Alt+KP_5"
-#define defgKeyWinSnapMoveN             XK_KP_Up, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_8"
-#define defgKeyWinSnapMoveNE            XK_KP_Prior, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_9"
-#define defgKeyWinSnapMoveE             XK_KP_Right, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_6"
-#define defgKeyWinSnapMoveSE            XK_KP_Next, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_3"
-#define defgKeyWinSnapMoveS             XK_KP_Down, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_2"
-#define defgKeyWinSnapMoveSW            XK_KP_End, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_1"
-#define defgKeyWinSnapMoveW             XK_KP_Left, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_4"
-#define defgKeyWinSnapMoveNW            XK_KP_Home, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_7"
-#define defgKeyWinSmartPlace            XK_KP_Begin, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_5"
+#define defgKeyWinArrangeN              XK_KP_Up, kfCtrl+kfAlt, "Ctrl+Alt+KP_Up"
+#define defgKeyWinArrangeNE             XK_KP_Prior, kfCtrl+kfAlt, "Ctrl+Alt+KP_Prior"
+#define defgKeyWinArrangeE              XK_KP_Right, kfCtrl+kfAlt, "Ctrl+Alt+KP_Right"
+#define defgKeyWinArrangeSE             XK_KP_Next, kfCtrl+kfAlt, "Ctrl+Alt+KP_Next"
+#define defgKeyWinArrangeS              XK_KP_Down, kfCtrl+kfAlt, "Ctrl+Alt+KP_Down"
+#define defgKeyWinArrangeSW             XK_KP_End, kfCtrl+kfAlt, "Ctrl+Alt+KP_End"
+#define defgKeyWinArrangeW              XK_KP_Left, kfCtrl+kfAlt, "Ctrl+Alt+KP_Left"
+#define defgKeyWinArrangeNW             XK_KP_Home, kfCtrl+kfAlt, "Ctrl+Alt+KP_Home"
+#define defgKeyWinArrangeC              XK_KP_Begin, kfCtrl+kfAlt, "Ctrl+Alt+KP_Begin"
+#define defgKeyWinTileLeft              0, 0, ""
+#define defgKeyWinTileRight             0, 0, ""
+#define defgKeyWinTileTop               0, 0, ""
+#define defgKeyWinTileBottom            0, 0, ""
+#define defgKeyWinTileTopLeft           0, 0, ""
+#define defgKeyWinTileTopRight          0, 0, ""
+#define defgKeyWinTileBottomLeft        0, 0, ""
+#define defgKeyWinTileBottomRight       0, 0, ""
+#define defgKeyWinTileCenter            0, 0, ""
+#define defgKeyWinSmartPlace            XK_KP_Begin, kfCtrl+kfAlt+kfShift, "Ctrl+Alt+Shift+KP_Begin"
 #define defgKeySysSwitchNext            XK_Tab, kfAlt, "Alt+Tab"
 #define defgKeySysSwitchLast            XK_Tab, kfAlt+kfShift, "Alt+Shift+Tab"
 #define defgKeySysSwitchClass           XK_grave, kfAlt, "Alt+grave"
@@ -52,6 +53,7 @@
 #define defgKeySysMenu                  XK_Escape, kfCtrl, "Ctrl+Esc"
 #define defgKeySysWindowList            XK_Escape, kfCtrl+kfAlt, "Alt+Ctrl+Esc"
 #define defgKeySysWinListMenu           0, 0, ""
+#define defgKeySysKeyboardNext          0, 0, ""
 #define defgKeySysAddressBar            ' ', kfAlt+kfCtrl, "Alt+Ctrl+Space"
 #define defgKeyWinMenu                  ' ', kfAlt, "Alt+Space"
 #define defgKeySysWorkspacePrev         XK_Left, kfAlt+kfCtrl, "Alt+Ctrl+Left"
@@ -70,8 +72,8 @@
 #define defgKeySysWorkspace8            '8', kfAlt+kfCtrl, "Alt+Ctrl+8"
 #define defgKeySysWorkspace9            '9', kfAlt+kfCtrl, "Alt+Ctrl+9"
 #define defgKeySysWorkspace10           '0', kfAlt+kfCtrl, "Alt+Ctrl+0"
-#define defgKeySysWorkspace11           '-', kfAlt+kfCtrl, "Alt+Ctrl+bracketleft"
-#define defgKeySysWorkspace12           '=', kfAlt+kfCtrl, "Alt+Ctrl+bracketright"
+#define defgKeySysWorkspace11           '-', kfAlt+kfCtrl, "Alt+Ctrl+minus"
+#define defgKeySysWorkspace12           '=', kfAlt+kfCtrl, "Alt+Ctrl+equal"
 #define defgKeySysWorkspace1TakeWin     '1', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+1"
 #define defgKeySysWorkspace2TakeWin     '2', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+2"
 #define defgKeySysWorkspace3TakeWin     '3', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+3"
@@ -82,13 +84,13 @@
 #define defgKeySysWorkspace8TakeWin     '8', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+8"
 #define defgKeySysWorkspace9TakeWin     '9', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+9"
 #define defgKeySysWorkspace10TakeWin    '0', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+0"
-#define defgKeySysWorkspace11TakeWin    '-', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+bracketleft"
-#define defgKeySysWorkspace12TakeWin    '=', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+bracketright"
+#define defgKeySysWorkspace11TakeWin    '-', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+minus"
+#define defgKeySysWorkspace12TakeWin    '=', kfAlt+kfCtrl+kfShift, "Alt+Ctrl+Shift+equal"
 #define defgKeySysTileVertical          XK_F2, kfAlt+kfShift, "Alt+Shift+F2"
 #define defgKeySysTileHorizontal        XK_F3, kfAlt+kfShift, "Alt+Shift+F3"
 #define defgKeySysCascade               XK_F4, kfAlt+kfShift, "Alt+Shift+F4"
 #define defgKeySysArrange               XK_F5, kfAlt+kfShift, "Alt+Shift+F5"
-#define defgKeySysArrangeIcons          XK_F6, kfAlt+kfShift, "Alt+Shift+F8"
+#define defgKeySysArrangeIcons          XK_F8, kfAlt+kfShift, "Alt+Shift+F8"
 #define defgKeySysMinimizeAll           XK_F9, kfAlt+kfShift, "Alt+Shift+F9"
 #define defgKeySysHideAll               XK_F11, kfAlt+kfShift, "Alt+Shift+F11"
 #define defgKeySysUndoArrange           XK_F7, kfAlt+kfShift, "Alt+Shift+F7"
@@ -101,7 +103,6 @@
 
 #define DEF_WMKEY(k) extern WMKey k
 #define IS_WMKEY(k,m,b) b.eq(k,m)
-#define GRAB_WMKEY(k)   grabVKey(k.key, k.mod)
 #define KEY_NAME(k)     k.name
 #endif
 
@@ -109,12 +110,12 @@ DEF_WMKEY(gMouseWinMove);
 DEF_WMKEY(gMouseWinSize);
 DEF_WMKEY(gMouseWinRaise);
 DEF_WMKEY(gMouseWinLower);
+DEF_WMKEY(gMouseWinTabbing);
 
 DEF_WMKEY(gKeyWinRaise);
 DEF_WMKEY(gKeyWinOccupyAll);
 DEF_WMKEY(gKeyWinLower);
 DEF_WMKEY(gKeyWinClose);
-DEF_WMKEY(gKeyWinKill);
 DEF_WMKEY(gKeyWinRestore);
 DEF_WMKEY(gKeyWinPrev);
 DEF_WMKEY(gKeyWinNext);
@@ -136,14 +137,15 @@ DEF_WMKEY(gKeyWinArrangeSW);
 DEF_WMKEY(gKeyWinArrangeW);
 DEF_WMKEY(gKeyWinArrangeNW);
 DEF_WMKEY(gKeyWinArrangeC);
-DEF_WMKEY(gKeyWinSnapMoveN);
-DEF_WMKEY(gKeyWinSnapMoveNE);
-DEF_WMKEY(gKeyWinSnapMoveE);
-DEF_WMKEY(gKeyWinSnapMoveSE);
-DEF_WMKEY(gKeyWinSnapMoveS);
-DEF_WMKEY(gKeyWinSnapMoveSW);
-DEF_WMKEY(gKeyWinSnapMoveW);
-DEF_WMKEY(gKeyWinSnapMoveNW);
+DEF_WMKEY(gKeyWinTileLeft);
+DEF_WMKEY(gKeyWinTileRight);
+DEF_WMKEY(gKeyWinTileTop);
+DEF_WMKEY(gKeyWinTileBottom);
+DEF_WMKEY(gKeyWinTileTopLeft);
+DEF_WMKEY(gKeyWinTileTopRight);
+DEF_WMKEY(gKeyWinTileBottomLeft);
+DEF_WMKEY(gKeyWinTileBottomRight);
+DEF_WMKEY(gKeyWinTileCenter);
 DEF_WMKEY(gKeyWinSmartPlace);
 DEF_WMKEY(gKeyWinMenu);
 DEF_WMKEY(gKeySysSwitchNext);
@@ -160,6 +162,7 @@ DEF_WMKEY(gKeySysDialog);
 DEF_WMKEY(gKeySysMenu);
 DEF_WMKEY(gKeySysWindowList);
 DEF_WMKEY(gKeySysWinListMenu);
+DEF_WMKEY(gKeySysKeyboardNext);
 DEF_WMKEY(gKeySysAddressBar);
 DEF_WMKEY(gKeySysWorkspacePrev);
 DEF_WMKEY(gKeySysWorkspaceNext);
