@@ -1,6 +1,6 @@
 /*
  *  FDOmenu - Menu code generator for icewm
- *  Copyright (C) 2015-2018 Eduard Bloch
+ *  Copyright (C) 2015-2022 Eduard Bloch
  *
  *  Inspired by icewm-menu-gnome2 and Freedesktop.org specifications
  *  Using pure glib/gio code and a built-in menu structure instead
@@ -15,15 +15,21 @@
  *  - overhauled program design and menu construction code, added sub-category handling
  */
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
 #include "config.h"
 #include "base.h"
 #include "sysdep.h"
 #include "intl.h"
 #include "appnames.h"
 #include <ctype.h>
-#include "debug.h"
 
-char const *ApplicationName(nullptr);
+#include <stack>
+#include <string>
+
+char const* ApplicationName;
 
 #ifndef LPCSTR // mind the MFC
 // easier to read...
@@ -44,6 +50,15 @@ bool no_sep_others = false;
 bool no_sub_cats = false;
 bool generic_name = false;
 bool right_to_left = false;
+bool flat_output = false;
+bool match_in_section = false;
+bool match_in_section_only = false;
+
+auto substr_filter = "";
+auto substr_filter_nocase = "";
+auto flat_sep = " / ";
+char* terminal_command;
+char* terminal_option;
 
 template<typename T, void TFreeFunc(T)>
 struct auto_raii {
@@ -95,7 +110,56 @@ tListMeta* lookup_category(LPCSTR key)
     return ret;
 }
 
+std::stack<std::string> flat_pfxes;
 
+const char* flat_pfx()
+{
+    if (!flat_output || flat_pfxes.empty())
+        return "";
+    return flat_pfxes.top().c_str();
+}
+
+void flat_add_level(const char *name)
+{
+    if (!flat_output)
+        return;
+    if (flat_pfxes.empty())
+        flat_pfxes.push(std::string(name) + flat_sep);
+    else
+        flat_pfxes.push(flat_pfxes.top() + name + flat_sep);
+}
+
+void flat_drop_level()
+{
+    flat_pfxes.pop();
+}
+
+bool filter_matched(const char* title, const char* section_prefix)
+{
+    if (!match_in_section)
+        section_prefix = nullptr;
+    if (match_in_section_only)
+        title = nullptr;
+
+    bool hasFilter(substr_filter && *substr_filter),
+            hasIfilter (substr_filter_nocase && *substr_filter_nocase);
+
+    if (hasFilter) {
+         if(title && strstr(title, substr_filter))
+             return true;
+         if(section_prefix && strstr(section_prefix, substr_filter))
+             return true;
+    }
+#ifdef __GNU_LIBRARY__
+    if (hasIfilter) {
+        if(title && strcasestr(title, substr_filter_nocase))
+            return true;
+        if(section_prefix && strcasestr(section_prefix, substr_filter_nocase))
+            return true;
+    }
+#endif
+    return ! (hasFilter || hasIfilter);
+}
 
 struct t_menu_node;
 extern t_menu_node root;
@@ -116,7 +180,7 @@ public:
         int count, level;
         t_menu_node* print_separated;
     };
-    static gboolean print_node(gpointer key, gpointer value, gpointer pr_meta) {
+    static gboolean print_node(gpointer /*key*/, gpointer value, gpointer pr_meta) {
         ((t_menu_node*) value)->print((t_print_meta*) pr_meta);
         return FALSE;
     }
@@ -127,20 +191,23 @@ public:
 
         if (!store)
         {
-            if (title && progCmd) {
+            if (title && progCmd &&
+                    filter_matched(title, flat_pfxes.empty()
+                                   ? nullptr : flat_pfxes.top().c_str()))
+            {
                 if (ctx->count == 0 && add_sep_before)
                     puts("separator");
                 if (nonempty(generic) && !strcasestr(title, generic)) {
                     if (right_to_left) {
-                        printf("prog \"(%s) %s\" %s %s\n",
-                                generic, title, meta->icon, progCmd);
+                        printf("prog \"(%s) %s%s\" %s %s\n",
+                                generic, flat_pfx(), title, meta->icon, progCmd);
                     } else {
-                        printf("prog \"%s (%s)\" %s %s\n",
-                                title, generic, meta->icon, progCmd);
+                        printf("prog \"%s%s (%s)\" %s %s\n",
+                                flat_pfx(), title, generic, meta->icon, progCmd);
                     }
                 }
                 else
-                    printf("prog \"%s\" %s %s\n", title, meta->icon, progCmd);
+                    printf("prog \"%s%s\" %s %s\n", flat_pfx(), title, meta->icon, progCmd);
             }
             ctx->count++;
             return;
@@ -160,23 +227,32 @@ public:
             if (ctx->count == 0 && add_sep_before)
                 puts("separator");
             ctx->count++;
-            printf("menu \"%s\" %s {\n", title, meta->icon);
+            if (flat_output)
+                flat_add_level(title);
+            else
+                printf("menu \"%s\" %s {\n", title, meta->icon);
         }
         ctx->level++;
         g_tree_foreach(store, print_node, ctx);
         if (ctx->level == 1 && ctx->print_separated)
         {
+            fflush(stdout);
             puts("separator");
             no_sep_others = true;
             ctx->print_separated->print(ctx);
         }
         ctx->level--;
-        if (ctx->level > 0)
+        if (ctx->level > 0) {
+            if (flat_output)
+                flat_drop_level();
+            else {
 #ifndef DEBUG
-            puts("}");
+                puts("}");
 #else
-            printf("# end of menu \"%s\"\n}\n", title);
+                printf("# end of menu \"%s\"\n}\n", title);
 #endif
+            }
+        }
         if (add_sep_after && ctx->level == 0 && ctx->count > 0)
             puts("separator");
 
@@ -445,10 +521,8 @@ struct t_menu_node_app : t_menu_node
 
         if (bUseSimplifiedCmd && !bForTerminal) // best case
             progCmd = cmdMod;
-    #ifdef XTERMCMD
-        else if (bForTerminal && bUseSimplifiedCmd)
-            progCmd = g_strjoin(" ", QUOTE(XTERMCMD), "-e", cmdMod, NULL);
-    #endif
+        else if (bForTerminal && nonempty(terminal_command))
+            progCmd = g_strjoin(" ", terminal_command, "-e", cmdMod, NULL);
         else
             // not simple command or needs a terminal started via launcher callback, or both
             progCmd = g_strdup_printf("%s \"%s\"", ApplicationName, dinfo.d_file);
@@ -669,6 +743,12 @@ static void init() {
         // enforce non-const since we are not destroying that data ever, no key_destroy_func set!
         g_hash_table_insert(meta_lookup_data, (gpointer) what.key, &what);
     }
+
+    const char* terminals[] = { terminal_option, getenv("TERMINAL"), TERM,
+                                "urxvt", "alacritty", "roxterm", "xterm" };
+    for (auto term : terminals)
+        if (term && (terminal_command = path_lookup(term)) != nullptr)
+            break;
 }
 
 static void help(LPCSTR home, LPCSTR dirs, FILE* out, int xit) {
@@ -677,11 +757,18 @@ static void help(LPCSTR home, LPCSTR dirs, FILE* out, int xit) {
             "OPTIONS:\n"
             "-g, --generic\tInclude GenericName in parentheses of progs\n"
             "-o, --output=FILE\tWrite the output to FILE\n"
+            "-t, --terminal=NAME\tUse NAME for a terminal that has '-e'\n"
             "--seps  \tPrint separators before and after contents\n"
             "--sep-before\tPrint separator before the contents\n"
             "--sep-after\tPrint separator only after contents\n"
             "--no-sep-others\tNo separation of the 'Others' menu point\n"
             "--no-sub-cats\tNo additional subcategories, just one level of menues\n"
+            "--flat\tDisplay all apps in one layer with category hints\n"
+            "--flat-sep STR\tCategory separator string used in flat mode (default: ' / ')\n"
+            "--match PAT\tDisplay only apps with title containing PAT\n"
+            "--imatch PAT\tLike --match but ignores the letter case\n"
+            "--match-sec\tApply --match or --imatch to apps AND sections\n"
+            "--match-osec\tApply --match or --imatch only to sections\n"
             "FILENAME\tAny .desktop file to launch its application Exec command\n"
             "This program also listens to environment variables defined by\n"
             "the XDG Base Directory Specification:\n"
@@ -750,8 +837,13 @@ int main(int argc, char** argv) {
     }
 
     for (char** pArg = argv + 1; pArg < argv + argc; ++pArg) {
-        if (is_version_switch(*pArg))
-            print_version_exit(VERSION);
+        if (is_version_switch(*pArg)) {
+            g_fprintf(stdout,
+                    "icewm-menu-fdo "
+                    VERSION
+                    ", Copyright 2015-2022 Eduard Bloch, 2017-2022 Bert Gijsbers.\n");
+            exit(0);
+        }
         else if (is_copying_switch(*pArg))
             print_copying_exit();
         else if (is_help_switch(*pArg))
@@ -766,6 +858,12 @@ int main(int argc, char** argv) {
             no_sep_others = true;
         else if (is_long_switch(*pArg, "no-sub-cats"))
             no_sub_cats = true;
+        else if (is_long_switch(*pArg, "flat"))
+            flat_output = no_sep_others = true;
+        else if (is_long_switch(*pArg, "match-sec"))
+            match_in_section = true;
+        else if (is_long_switch(*pArg, "match-osec"))
+            match_in_section = match_in_section_only = true;
         else if (is_switch(*pArg, "g", "generic-name"))
             generic_name = true;
         else {
@@ -783,6 +881,14 @@ int main(int argc, char** argv) {
                 if (expand)
                     delete[] expand;
             }
+            else if (GetArgument(value, "m", "match", pArg, argv + argc))
+                substr_filter = value;
+            else if (GetArgument(value, "M", "imatch", pArg, argv + argc))
+                substr_filter_nocase = value;
+            else if (GetArgument(value, "F", "flat-sep", pArg, argv + argc))
+                flat_sep = value;
+            else if (GetArgument(value, "t", "terminal", pArg, argv + argc))
+                terminal_option = value;
             else // unknown option
                 help(usershare, sysshare, stderr, EXIT_FAILURE);
         }
@@ -802,6 +908,7 @@ int main(int argc, char** argv) {
 
     if (nonempty(usershare) && usershare != getenv("XDG_DATA_HOME"))
         g_free(usershare);
+    delete[] terminal_command;
 
     return EXIT_SUCCESS;
 }
