@@ -2,8 +2,6 @@
 
 #include <unistd.h>
 #include <signal.h>
-#include <time.h>
-#include <sys/stat.h>
 #include <sys/resource.h>
 #include <stdlib.h>
 #include <X11/Xatom.h>
@@ -11,6 +9,7 @@
 #include "yprefs.h"
 #include "ytimer.h"
 #include "udir.h"
+#include "ascii.h"
 #include "intl.h"
 #include "appnames.h"
 
@@ -49,6 +48,7 @@ public:
     void add(const char* name, const char* value, bool append);
     bool become(bool replace = false);
     void update(bool force = false);
+    void printOptions();
     void sendQuit()    { sendClientMessage(_XA_ICEWMBG_QUIT);    }
     void sendRestart() { sendClientMessage(_XA_ICEWMBG_RESTART); }
     void sendShuffle() { sendClientMessage(_XA_ICEWMBG_SHUFFLE); }
@@ -138,6 +138,7 @@ private:
     YArray<int> sequence;
     lazy<YTimer> cycleTimer;
     lazy<YTimer> checkTimer;
+    lazy<YTimer> randrTimer;
 
     Atom _XA_XROOTPMAP_ID;
     Atom _XA_XROOTCOLOR_PIXEL;
@@ -194,6 +195,11 @@ Background::Background(int *argc, char ***argv, bool verb):
     catchSignal(SIGQUIT);
     catchSignal(SIGUSR1);
     catchSignal(SIGUSR2);
+
+#ifdef CONFIG_XRANDR
+    if (xrandr.supported)
+        XRRSelectInput(xapp->display(), root(), RRScreenChangeNotifyMask);
+#endif
 }
 
 upath Background::getThemeDir() {
@@ -256,20 +262,34 @@ Atom Background::atom(const char* name) const {
 }
 
 static bool hasImageExt(const char* name) {
-    size_t len = strlen(name);
-    if (len >= 5 && name[len - 4] == '.') {
-        const char* ext = &name[len - 3];
-        if (3 == strspn(ext, "jpngxmJPNGXM"))
-            return true;
-/* #ifdef CONFIG_LIBRSVG
-        if (0 == strcasecmp(ext, "svg"))
-            return true;
-* #endif */
-    }
-    else if (len >= 6 && name[len - 5] == '.') {
-        const char* ext = &name[len - 4];
-        if (0 == strcasecmp(ext, "jpeg"))
-            return true;
+    const char* dot = strrchr(name, '.');
+    if (dot) {
+        const char* ext = dot + 1;
+        size_t len = strlen(ext);
+        if (len > 2 && len < 5) {
+            char low[8];
+            for (int i = 0; i < int(len); ++i)
+                low[i] = ASCII::toLower(ext[i]);
+            low[int(len)] = '\0';
+            if (len == 3) {
+                if (strcmp(low, "xpm") == 0 ||
+                    strcmp(low, "png") == 0 ||
+                    strcmp(low, "jpg") == 0 ||
+                    (strcmp(low, "jxl") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "jp2") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "raw") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "svg") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "tga") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "tif") == 0 && YImage::supportsFormat(low)))
+                    return true;
+            }
+            if (len == 4) {
+                if (strcmp(low, "jpeg") == 0 ||
+                    (strcmp(low, "tiff") == 0 && YImage::supportsFormat(low)) ||
+                    (strcmp(low, "webp") == 0 && YImage::supportsFormat(low)))
+                    return true;
+            }
+        }
     }
     return false;
 }
@@ -416,6 +436,13 @@ bool Background::handleTimer(YTimer* timer) {
             syncWM();
         update(true);
     }
+    else if (timer == randrTimer) {
+        if (desktop->updateXineramaInfo(desktopWidth, desktopHeight)) {
+            if (verbose) tlog("desktop change by RandR event: %ux%u",
+                              desktopWidth, desktopHeight);
+            update(true);
+        }
+    }
     return false;
 }
 
@@ -721,6 +748,9 @@ void Background::changeBackground(bool force) {
                            (unsigned char *) &tPixmap);
             changeProperty(_XA_XROOTCOLOR_PIXEL, XA_CARDINAL,
                            (unsigned char *) &tPixel);
+
+            if (currentTransparencyPixmap == null && tPixmap == bPixmap)
+                currentTransparencyPixmap = back;
         }
     }
     XClearWindow(display(), window());
@@ -769,12 +799,18 @@ bool Background::filterEvent(const XEvent &xev) {
         else if (xev.xproperty.atom == _XA_NET_DESKTOP_GEOMETRY) {
             unsigned w = desktopWidth, h = desktopHeight;
             getDesktopGeometry();
-            update(w != desktopWidth || h != desktopHeight);
+            if (w != desktopWidth || h != desktopHeight) {
+                if (verbose) tlog("desktop change by geometry property: %ux%u",
+                                  desktopWidth, desktopHeight);
+                checkTimer->setTimer(100L, this, true);
+            }
         }
         else if (xev.xproperty.atom == _XA_NET_WORKAREA) {
-            unsigned w = desktopWidth, h = desktopHeight;
-            desktop->updateXineramaInfo(desktopWidth, desktopHeight);
-            update(w != desktopWidth || h != desktopHeight);
+            if (desktop->updateXineramaInfo(desktopWidth, desktopHeight)) {
+                if (verbose) tlog("desktop change by workarea property: %ux%u",
+                                  desktopWidth, desktopHeight);
+                checkTimer->setTimer(100L, this, true);
+            }
         }
         else if (xev.xproperty.atom == _XA_ICEWMBG_PID) {
             int pid = getBgPid();
@@ -804,6 +840,12 @@ bool Background::filterEvent(const XEvent &xev) {
             return true;
         }
     }
+#ifdef CONFIG_XRANDR
+    else if (xrandr.isEvent(xev.type, RRScreenChangeNotify)) {
+        randrTimer->setTimer(0L, this, true);
+        return true;
+    }
+#endif
 
     return YXApplication::filterEvent(xev);
 }
@@ -868,6 +910,32 @@ bool Background::become(bool replace) {
     return pid == mypid;
 }
 
+void Background::printOptions() {
+    for (cfoption* o = icewmbg_prefs; o && o->type; ++o) {
+        switch (o->type) {
+            case cfoption::CF_BOOL:
+                printf("%-26s = %d\n", o->name, *o->v.b.bool_value);
+                break;
+            case cfoption::CF_INT:
+                printf("%-26s = %d\n", o->name, *o->v.i.int_value);
+                break;
+            default:
+                /*ignore*/
+                break;
+        }
+    }
+    printf("%-26s = ", "DesktopBackgroundImage");
+    int i = 0;
+    for (auto s : backgroundImages)
+        printf("%s%s", i++ ? ", " : "", s.c_str());
+    printf("\n");
+    printf("%-26s = ", "DesktopTransparencyImage");
+    i = 0;
+    for (auto s : transparencyImages)
+        printf("%s%s", i++ ? ", " : "", s.c_str());
+    printf("\n");
+}
+
 static const char* get_help_text() {
     return _(
     "Usage: icewmbg [OPTIONS]\n"
@@ -956,7 +1024,7 @@ static void bgParse(const char* name, const char* value) {
     for (int i = 0; str.splitall(',', &opt, &str); ++i) {
         if (i + 1 < ICEBG_MAX_ARGS) {
             if ((opt = opt.trim()).nonempty()) {
-                globalBg->add(name, opt, 0 < i);
+                globalBg->add(name, opt, true);
             }
         }
     }
@@ -1014,26 +1082,46 @@ static bool sendMessage(const char* displayArg, const char* text) {
     return success;
 }
 
+static void appendFile(YArray<const char*>& image, const char* value) {
+    if (nonempty(value)) {
+        if (strchr("$/~", *value) == nullptr && access(value, R_OK) == 0) {
+            char buf[1234] = "";
+            char* pwd = getcwd(buf, sizeof buf);
+            if (pwd == buf && strlen(pwd) + strlen(value) + 2 < sizeof buf) {
+                strlcat(buf, "/", sizeof buf);
+                strlcat(buf, value, sizeof buf);
+                image += newstr(buf);
+            }
+        } else {
+            image += value;
+        }
+    }
+}
+
 int main(int argc, char **argv) {
     ApplicationName = my_basename(*argv);
 
+    bool daemonize = false;
     bool sendRestart = false;
     bool sendQuit = false;
+    bool postpref = false;
     bool replace = false;
     bool shuffle = false;
     bool verbose = false;
     const char* overrideTheme = nullptr;
     const char* configFile = nullptr;
-    const char* image = nullptr;
-    const char* color = nullptr;
-    const char* trans = nullptr;
-    const char* semis = nullptr;
+    typedef YArray<const char*> ArgVector;
+    ArgVector image;
+    ArgVector color;
+    ArgVector trans;
+    ArgVector semis;
     const char* center = nullptr;
     const char* scaled = nullptr;
     const char* multi = nullptr;
     const char* cycle = nullptr;
     const char* shuffleArg = nullptr;
     const char* displayArg = nullptr;
+    const char* outputArg = nullptr;
 
     for (char **arg = argv + 1; arg < argv + argc; ++arg) {
         if (**arg == '-') {
@@ -1042,6 +1130,9 @@ int main(int argc, char **argv) {
                 sendRestart = true;
                 **arg = '\0';
             }
+            else if (is_switch(*arg, "f", "fork")) {
+                daemonize = true;
+            }
             else if (is_switch(*arg, "q", "quit")) {
                 sendQuit = true;
             }
@@ -1052,6 +1143,10 @@ int main(int argc, char **argv) {
                     (arg + 1 == argv + argc || strcspn(arg[1], "01")))
             {
                 shuffle = true;
+                shuffleArg = "1";
+            }
+            else if (is_long_switch(*arg, "postpreferences")) {
+                postpref = true;
             }
             else if (is_help_switch(*arg)) {
                 print_help_xit();
@@ -1072,16 +1167,16 @@ int main(int argc, char **argv) {
                 configFile = value;
             }
             else if (GetArgument(value, "i", "image", arg, argv + argc)) {
-                image = value;
+                appendFile(image, value);
             }
             else if (GetArgument(value, "k", "color", arg, argv + argc)) {
-                color = value;
+                color += value;
             }
             else if (GetArgument(value, "s", "semis", arg, argv + argc)) {
-                semis = value;
+                appendFile(semis, value);
             }
             else if (GetArgument(value, "x", "trans", arg, argv + argc)) {
-                trans = value;
+                trans += value;
             }
             else if (GetArgument(value, "e", "center", arg, argv + argc)) {
                 center = value;
@@ -1102,12 +1197,24 @@ int main(int argc, char **argv) {
             else if (GetArgument(value, "d", "display", arg, argv + argc)) {
                 displayArg = value;
             }
+            else if (GetArgument(value, "o", "output", arg, argv + argc)) {
+                outputArg = value;
+            }
             else if (GetArgument(value, "C", "copying", arg, argv + argc)) {
                 /*ignore*/;
             }
             else if (*arg)
                 warn(_("Unrecognized option '%s'."), *arg);
         }
+        else if (hasImageExt(*arg)) {
+            appendFile(image, *arg);
+        }
+        else {
+            upath path(*arg + (**arg == '!'));
+            path = path.expand();
+            if (path.dirExists() && path.isSearchable())
+                appendFile(image, *arg);
+        }
     }
 
     if (getpriority(PRIO_PROCESS, 0) < 5) {
@@ -1115,6 +1222,8 @@ int main(int argc, char **argv) {
             /*ignore*/;
         }
     }
+    if (outputArg)
+        upath::redirectOutput(outputArg);
 
     if (sendQuit) {
         sendMessage(displayArg, "_ICEWMBG_QUIT");
@@ -1130,34 +1239,28 @@ int main(int argc, char **argv) {
     }
 
     Background bg(&argc, &argv, verbose);
-
-    if (bg.become(replace) == false) {
-        if (shuffle) {
-            bg.sendShuffle();
-            return 0;
-        }
-        warn(_("Cannot start, because another icewmbg is still running."));
-        return 1;
-    }
-
     globalBg = &bg;
     bgLoadConfig(configFile, overrideTheme);
-    if (image) {
+    if (image.nonempty()) {
         bg.clearBackgroundImages();
-        bgParse("DesktopBackgroundImage", image);
+        for (const char* arg : image)
+            bgParse("DesktopBackgroundImage", arg);
     }
-    if (color) {
+    if (color.nonempty()) {
         bg.clearBackgroundColors();
-        bgParse("DesktopBackgroundColor", color);
+        for (const char* arg : color)
+            bgParse("DesktopBackgroundColor", arg);
     }
-    if (semis) {
+    if (semis.nonempty()) {
         bg.clearTransparencyImages();
-        bgParse("DesktopTransparencyImage", semis);
+        for (const char* arg : semis)
+            bgParse("DesktopTransparencyImage", arg);
         bg.enableSemitransparency();
     }
-    if (trans) {
+    if (trans.nonempty()) {
         bg.clearTransparencyColors();
-        bgParse("DesktopTransparencyColor", trans);
+        for (const char* arg : trans)
+            bgParse("DesktopTransparencyColor", arg);
         bg.enableSemitransparency();
     }
     testFlag(&centerBackground, center);
@@ -1180,7 +1283,34 @@ int main(int argc, char **argv) {
 #endif
     }
 
+    if (postpref) {
+        bg.printOptions();
+    }
+    if (bg.become(replace) == false) {
+        if (shuffle) {
+            bg.sendShuffle();
+            return 0;
+        }
+        warn(_("Cannot start, because another icewmbg is still running."));
+        return 1;
+    }
+
     globalBg = nullptr;
+    if (image.nonempty())
+        image.clear();
+    if (color.nonempty())
+        color.clear();
+    if (semis.nonempty())
+        semis.clear();
+    if (trans.nonempty())
+        trans.clear();
+
+    if (daemonize) {
+        if (daemon(1, 0) == -1)
+            fail("daemon");
+        if (nonempty(outputArg))
+            upath::redirectOutput(outputArg);
+    }
 
     return bg.mainLoop();
 }
