@@ -61,7 +61,7 @@ static const char eventNames[][17] = {
     "MappingNotify",        // 34
     "GenericEvent",         // 35
 };
-static const char* eventName(int eventType) {
+const char* eventName(int eventType) {
     if (inrange(eventType, KeyPress, GenericEvent))
         return eventNames[eventType - KeyPress];
     return "UnknownEvent!";
@@ -247,6 +247,17 @@ void logKey(const XKeyEvent& xev) {
         boolStr(xev.same_screen));
 }
 
+void logMapping(const XMappingEvent& xev) {
+    tlog("window=0x%lX: mapping request=%s, keycode=%d, n=%d, serial=%lu, send=%s",
+        xev.window,
+        xev.request == MappingModifier ? "MappingModifier" :
+        xev.request == MappingKeyboard ? "MappingKeyboard" :
+        xev.request == MappingPointer ? "MappingPointer" : "?",
+        xev.first_keycode, xev.count,
+        (unsigned long) xev.serial,
+        boolStr(xev.send_event));
+}
+
 void logMapRequest(const XMapRequestEvent& xev) {
     tlog("window=0x%lX: mapRequest serial=%lu parent=0x%lX",
         xev.window,
@@ -409,6 +420,14 @@ void logRandrNotify(const XEvent& xev) {
 }
 #endif
 
+#else
+const char* eventName(int eventType) {
+    static char name[3];
+    name[0] = eventType / 10 % 10 + '0';
+    name[1] = eventType % 10 + '0';
+    name[2] = '\0';
+    return name;
+}
 #endif
 
 #if LOGEVENTS
@@ -419,7 +438,7 @@ void logRandrNotify(const XEvent& xev) {
 #pragma GCC diagnostic ignored "-Wcast-function-type"
 #endif
 
-void logEvent(const XEvent& xev) {
+void logEvent(const XEvent& xev, bool force) {
 #if LOGEVENTS
     typedef void (*fun)(const XEvent&);
     static void (*const loggers[])(const XEvent&) = {
@@ -457,11 +476,11 @@ void logEvent(const XEvent& xev) {
         (fun) logAny,               // 31 SelectionNotify
         (fun) logColormap,          // 32 ColormapNotify
         (fun) logClientMessage,     // 33 ClientMessage
-        (fun) logAny,               // 34 MappingNotify
+        (fun) logMapping,           // 34 MappingNotify
         (fun) logAny,               // 35 GenericEvent
     };
-    if (loggingEvents && size_t(xev.type) < sizeof loggedEvents &&
-        (loggedEventsInited || initLogEvents()) && loggedEvents[xev.type])
+    if (inrange(xev.type, 2, 35) && (force || (loggingEvents &&
+        (loggedEventsInited || initLogEvents()) && loggedEvents[xev.type])))
     {
         loggers[xev.type](xev);
     }
@@ -482,10 +501,10 @@ bool initLogEvents() {
         setLogEvent(ButtonPress, true);
         setLogEvent(ButtonRelease, true);
         // setLogEvent(MotionNotify, true);
-        // setLogEvent(EnterNotify, true);
-        // setLogEvent(LeaveNotify, true);
-        // setLogEvent(FocusIn, true);
-        // setLogEvent(FocusOut, true);
+        setLogEvent(EnterNotify, true);
+        setLogEvent(LeaveNotify, true);
+        setLogEvent(FocusIn, true);
+        setLogEvent(FocusOut, true);
         // setLogEvent(KeymapNotify, true);
         // setLogEvent(Expose, true);
         // setLogEvent(GraphicsExpose, true);
@@ -500,7 +519,7 @@ bool initLogEvents() {
         setLogEvent(ConfigureNotify, true);
         setLogEvent(ConfigureRequest, true);
         // setLogEvent(GravityNotify, true);
-        // setLogEvent(ResizeRequest, true);
+        setLogEvent(ResizeRequest, true);
         // setLogEvent(CirculateNotify, true);
         // setLogEvent(CirculateRequest, true);
         // setLogEvent(PropertyNotify, true);
@@ -509,7 +528,7 @@ bool initLogEvents() {
         // setLogEvent(SelectionNotify, true);
         // setLogEvent(ColormapNotify, true);
         setLogEvent(ClientMessage, true);
-        // setLogEvent(MappingNotify, true);
+        setLogEvent(MappingNotify, true);
         // setLogEvent(GenericEvent, true);
 
         loggedEventsInited = true;
