@@ -102,7 +102,8 @@ void msg(char const *msg, ...) {
 void tlog(char const *msg, ...) {
     timeval now;
     gettimeofday(&now, nullptr);
-    struct tm *loc = localtime(&now.tv_sec);
+    struct tm tmbuf = {};
+    struct tm *loc = localtime_r(&now.tv_sec, &tmbuf);
 
     fprintf(stderr, "%02d:%02d:%02d.%03u: %s: ", loc->tm_hour,
             loc->tm_min, loc->tm_sec,
@@ -218,8 +219,15 @@ size_t strlcat(char *dest, const char *from, size_t dest_size)
 }
 #endif
 
-char *newstr(char const *str) {
-    return (str != nullptr ? newstr(str, strlen(str)) : nullptr);
+char* newstr(const char* str) {
+    char* s = nullptr;
+    if (str) {
+        size_t len = strlen(str);
+        s = new char[len + 1];
+        if (s)
+            memcpy(s, str, len + 1);
+    }
+    return s;
 }
 
 char *newstr(char const *str, char const *delim) {
@@ -279,12 +287,6 @@ int strpcmp(char const * str, char const * pfx, char const * delim) {
     return (*pfx == '\0' && strchr(delim, *str) ? 0 : *str - *pfx);
 }
 
-char const * strnxt(const char * str, const char * delim) {
-    str+= strcspn(str, delim);
-    str+= strspn(str, delim);
-    return str;
-}
-
 #ifndef HAVE_MEMRCHR
 void* memrchr(const void* ptr, char chr, size_t num) {
     char* str = (char *) ptr;
@@ -305,6 +307,10 @@ char* tokens::operator++() {
     return tok = strtok_r(nullptr, sep, &save);
 }
 
+bool tokens::operator==(const char* str) const {
+    return strcmp(tok, str) == 0;
+}
+
 bool GetShortArgument(char* &ret, const char *name, char** &argpp, char **endpp)
 {
     unsigned int alen = strlen(name);
@@ -619,20 +625,25 @@ char* path_lookup(const char* name) {
         return isExeFile(name) ? newstr(name) : nullptr;
     }
 
-    strp env = newstr(getenv("PATH"));
+    char* env = getenv("PATH");
     if (env == nullptr)
         return nullptr;
 
     const size_t namlen = strlen(name);
 
-    for (tokens directory(env, ":"); directory; ++directory) {
-        size_t dirlen = strlen(directory);
+    for (char* str = env, *end; *str; str = (end + (*end == ':'))) {
+        for (end = str; *end != ':' && *++end; );
+        size_t dirlen = end - str;
         size_t length = dirlen + namlen + 3;
         const size_t bufsize = 1234;
         if (length < bufsize) {
             char filebuf[bufsize];
-            snprintf(filebuf, bufsize, "%s/%s",
-                     dirlen ? directory : ".", name);
+            if (dirlen) {
+                memcpy(filebuf, str, dirlen);
+                if (filebuf[dirlen - 1] != '/')
+                    filebuf[dirlen++] = '/';
+            }
+            memcpy(filebuf + dirlen, name, namlen + 1);
             if (isExeFile(filebuf)) {
                 return newstr(filebuf);
             }
@@ -649,7 +660,7 @@ const char* getprogname() {
 
 // get path of executable.
 char* progpath() {
-#ifdef __linux__
+#if defined(__linux__) && defined(_GNU_SOURCE) && !defined(__ANDROID__)
     char* path = program_invocation_name;
     bool fail = (isEmpty(path) || !isExeFile(path));
     if (fail) {
@@ -695,7 +706,7 @@ void show_backtrace(const int limit) {
     timeval now;
     gettimeofday(&now, nullptr);
     struct tm *loc = localtime(&now.tv_sec);
-    unsigned msec = (unsigned)(now.tv_usec / 1000);
+    unsigned msec = unsigned(now.tv_usec / 1000);
 
     fprintf(stderr, "%02d:%02d:%02d.%03u: %s:\n", loc->tm_hour, loc->tm_min,
             loc->tm_sec, msec, "backtrace"); fflush(stderr);
@@ -717,7 +728,8 @@ void show_backtrace(const int limit) {
             int lineCount = 0;
             while (fgets(line, linesize, fp)) {
                 ++lineCount;
-                if (strncmp(line, "?? ??:0", 7)) {
+                if (strncmp(line, "?? ??:0", 7) &&
+                    strncmp(line, "show_backtrace(", 15)) {
                     fputs(line, stderr);
                 }
             }
