@@ -20,17 +20,23 @@
 
 #include "intl.h"
 
-AboutDlg::AboutDlg():
-    fSizeable(nullptr)
+AboutDlg::AboutDlg(YActionListener* al):
+    fSizeable(nullptr),
+    fListener(al)
 {
     const char version[] = "IceWM " VERSION " (" HOSTOS "/" HOSTCPU ")";
-    mstring copyright = mstring("Copyright ")
-                      + _("(C)") + " 1997-2012 Marko Macek, "
-                      + _("(C)") + " 2001 Mathias Hasselmann";
+    mstring copyhead = mstring("Copyright\t");
+    mstring copysymb = mstring(_("(C)"));
+    mstring copyright = copyhead
+                      + copysymb + " 1997-2012 Marko Macek, "
+                      + copysymb + " 2001 Mathias Hasselmann,  \n\t"
+                      + copysymb + " 2016-2023 Bert Gijsbers. \n";
 
     Ladder* ladder = new Ladder();
     *ladder += label(version);
-    *ladder += label(copyright);
+    YLabel* copylabel = label(copyright);
+    copylabel->multiline(true);
+    *ladder += copylabel;
     *ladder += new Spacer(1, 20);
 
     Table* table = new Table();
@@ -55,11 +61,11 @@ AboutDlg::AboutDlg():
              YImage::renderName(), doubleBuffer ? _("DoubleBuffer") : "",
              xapp->alpha() ? _("AlphaBlending") : "");
     *table += new Row(label(_("Renderer:")), label(text));
+    if (rightToLeft)
+        table->swapColumns();
 
     fSizeable = new Padder(ladder, 20, 20);
-    fSizeable->setPosition(0, 0);
-    fSizeable->realize();
-    setSize(fSizeable->width(), fSizeable->height());
+    fSizeable->layout(this);
 
     setTitle("About");
     setWindowTitle(_("icewm - About"));
@@ -83,23 +89,15 @@ YLabel* AboutDlg::label(const char* text) {
 }
 
 void AboutDlg::showFocused() {
-    int dx, dy;
-    unsigned dw, dh;
-    desktop->getScreenGeometry(&dx, &dy, &dw, &dh);
-
-    if (getFrame() == nullptr)
-        manager->manageClient(handle(), false);
-    if (getFrame() != nullptr) {
-        getFrame()->setNormalPositionOuter(
-            dx + int(dw / 2 - getFrame()->width() / 2),
-            dy + int(dh / 2 - getFrame()->height() / 2));
-        getFrame()->activateWindow(true);
-    }
+    center();
+    become();
 }
 
 void AboutDlg::handleClose() {
-    if (!getFrame()->isHidden())
+    if (getFrame()->isHidden() == false)
         getFrame()->wmHide();
+    if (fListener)
+        fListener->actionPerformed(actionAboutClose);
 }
 
 // vim: set sw=4 ts=4 et:
