@@ -345,13 +345,6 @@ ref<YIcon> YWMApp::getDefaultAppIcon() {
     return defaultAppIcon;
 }
 
-CtrlAltDelete* YWMApp::getCtrlAltDelete() {
-    if (ctrlAltDelete == nullptr) {
-        ctrlAltDelete = new CtrlAltDelete(this, desktop);
-    }
-    return ctrlAltDelete;
-}
-
 AToolTip* YWMApp::newToolTip() {
     return new YToolTip;
 }
@@ -493,6 +486,23 @@ void YWMApp::initPointers() {
     }
 }
 
+void YWMApp::keyboardRemap() {
+    fixupPreferences();
+    for (KProgramIterType p = keyProgs.iterator(); ++p; ) {
+        p->parse();
+    }
+}
+
+void YWMApp::fixupPreferences() {
+    extern cfoption icewm_preferences[];
+    for (cfoption* op = icewm_preferences; op->type; ++op) {
+        if (op->type == cfoption::CF_KEY) {
+            WMKey* key = op->v.k.key_value;
+            xapp->unshift(&key->key, &key->mod);
+        }
+    }
+}
+
 void LogoutMenu::updatePopup() {
     if (itemCount())
         return;
@@ -512,9 +522,10 @@ void LogoutMenu::updatePopup() {
                 addItem(_("Re_boot"), -2, null, actionReboot, "reboot");
             if (canShutdown(Shutdown))
                 addItem(_("Shut_down"), -2, null, actionShutdown, "shutdown");
-            if (couldRunCommand(suspendCommand))
+            if (canSuspend())
                 addItem(_("_Sleep mode"), -2, null, actionSuspend, "suspend");
-
+            if (canHibernate())
+                addItem(_("_Hibernate"), -2, null, actionHibernate, "hibernate");
             if (itemCount() != oldItemCount)
                 addSeparator();
 
@@ -881,21 +892,37 @@ bool YWMApp::mapClientByPid(const char* resource, long pid) {
     if (isEmpty(resource))
         return false;
 
-    bool found = false;
+    YFrameWindow* bestFrame = nullptr;
+    YFrameClient* bestTab = nullptr;
+    const int active = manager->activeWorkspace();
+    long tmp = 0;
 
     for (YFrameIter frame = manager->focusedIterator(); ++frame; ) {
-        long tmp = 0;
-        if (frame->client()->getNetWMPid(&tmp) && tmp == pid) {
-            if (frame->client()->classHint()->match(resource)) {
-                frame->setWorkspace(manager->activeWorkspace());
-                frame->activateWindow(true);
-                found = true;
-                break;
+        for (YFrameClient* tab : frame->clients()) {
+            if (tab->getNetWMPid(&tmp) && tmp == pid &&
+                tab->classHint()->match(resource)) {
+                if (bestFrame == nullptr ||
+                    (abs(frame->getWorkspace() - active) <
+                     abs(bestFrame->getWorkspace() - active))) {
+                    bestFrame = frame;
+                    bestTab = tab;
+                }
+                else if (frame == bestFrame && tab == frame->client())
+                    bestTab = tab;
             }
         }
+        if (bestFrame && bestFrame->getWorkspace() == active)
+            break;
+    }
+    if (bestFrame && bestTab) {
+        if (bestFrame->client() != bestTab)
+            bestFrame->selectTab(bestTab);
+        bestFrame->setWorkspace(active);
+        bestFrame->activateWindow(true);
+        return true;
     }
 
-    return found;
+    return false;
 }
 
 bool YWMApp::mapClientByResource(const char* resource, long *pid) {
@@ -935,6 +962,8 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
         doLogout(Shutdown);
     } else if (action == actionSuspend) {
         runCommand(suspendCommand);
+    } else if (action == actionHibernate) {
+        runCommand(hibernateCommand);
     } else if (action == actionReboot) {
         doLogout(Reboot);
     } else if (action == actionRestart) {
@@ -946,6 +975,15 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
 #endif
             restartClient(nullptr, nullptr);
     }
+    else if (action == actionIcewmbg) {
+        if (notifyParent && notifiedParent && kill(notifiedParent, SIGUSR2) == 0)
+            ;
+        else {
+            const char* bg[] = { ICEWMBGEXE, "-r", nullptr };
+            int pid = runProgram(bg[0], bg);
+            waitProgram(pid);
+        }
+    }
     else if (action == actionRestartXterm) {
         delete fRestartMsgBox;
         fRestartMsgBox = new YMsgBox(YMsgBox::mbBoth,
@@ -1011,6 +1049,12 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
     else if (action == actionUndoArrange) {
         manager->undoArrange();
     }
+    else if (action == actionSysDialog) {
+        if (ctrlAltDelete == nullptr)
+            ctrlAltDelete = new CtrlAltDelete(this, desktop);
+        if (ctrlAltDelete && ctrlAltDelete->visible() == false)
+            ctrlAltDelete->activate();
+    }
     else if (action == actionWindowList) {
         if (windowList->visible())
             windowList->handleClose();
@@ -1065,7 +1109,6 @@ void YWMApp::initFocusMode() {
 
     case FocusClick: /* click to focus */
         clickFocus = true;
-        // focusOnAppRaise = false;
         requestFocusOnAppRaise = true;
         raiseOnFocus = true;
         raiseOnClickClient = true;
@@ -1078,7 +1121,6 @@ void YWMApp::initFocusMode() {
 
     case FocusSloppy:  /* sloppy mouse focus */
         clickFocus = false;
-        // focusOnAppRaise = false;
         requestFocusOnAppRaise = true;
         raiseOnFocus = false;
         raiseOnClickClient = true;
@@ -1091,7 +1133,6 @@ void YWMApp::initFocusMode() {
 
     case FocusExplicit: /* explicit focus */
         clickFocus = true;
-        // focusOnAppRaise = false;
         requestFocusOnAppRaise = false;
         raiseOnFocus = false;
         raiseOnClickClient = false;
@@ -1104,7 +1145,6 @@ void YWMApp::initFocusMode() {
 
     case FocusStrict:  /* strict mouse focus */
         clickFocus = false;
-        // focusOnAppRaise = false;
         requestFocusOnAppRaise = false;
         raiseOnFocus = true;
         raiseOnClickClient = true;
@@ -1117,7 +1157,6 @@ void YWMApp::initFocusMode() {
 
     case FocusQuiet:  /* quiet sloppy focus */
         clickFocus = false;
-        // focusOnAppRaise = false;
         requestFocusOnAppRaise = false;
         raiseOnFocus = false;
         raiseOnClickClient = true;
@@ -1270,6 +1309,14 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName,
     if (focusMode != FocusCustom)
         initFocusMode();
 
+    if (post_preferences)
+        WMConfig::printPrefs(focusMode, wmapp_preferences);
+    if (show_extensions)
+        showExtensions();
+
+    fixupPreferences();
+
+    DEPRECATE(xrrDisable == true);
     DEPRECATE(warpPointer == true);
     DEPRECATE(focusRootWindow == true);
     DEPRECATE(replayMenuCancelClick == true);
@@ -1295,11 +1342,6 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName,
 
     initPointers();
 
-    if (post_preferences)
-        WMConfig::printPrefs(focusMode, wmapp_preferences);
-    if (show_extensions)
-        showExtensions();
-
     delete desktop;
 
     managerWindow = registerProtocols1(*argv, *argc);
@@ -1453,6 +1495,8 @@ int YWMApp::mainLoop() {
             notifyParent = false;
             fail("notify parent");
         }
+        else
+            kill(notifiedParent, SIGUSR2);
     }
 
     int rc = super::mainLoop();
@@ -1584,6 +1628,12 @@ static void print_usage(const char *argv0) {
              "  --rewrite-preferences  Update an existing preferences file.\n"
              "  --trace=conf,icon   Trace paths used to load configuration.\n"
              );
+    const char* output_preferences =
+             _(
+             "  -o, --output=FILE   Redirect all output to FILE.\n"
+             );
+    char* joint_preferences = cstrJoin(usage_preferences, output_preferences,
+                                       nullptr);
 
     printf(_("Usage: %s [OPTIONS]\n"
              "Starts the IceWM window manager.\n"
@@ -1616,11 +1666,12 @@ static void print_usage(const char *argv0) {
              "%s\n\n"),
              argv0,
              usage_client_id,
-             usage_preferences,
+             joint_preferences,
              usage_debug,
              PACKAGE_BUGREPORT[0] ? PACKAGE_BUGREPORT :
              PACKAGE_URL[0] ? PACKAGE_URL :
              "https://ice-wm.org/");
+    delete[] joint_preferences;
     exit(0);
 }
 
@@ -1741,12 +1792,16 @@ static void print_configured(const char *argv0) {
     exit(0);
 }
 
-static void loadStartup(const char* configFile)
+static void loadStartup(const char* configFile, const char* outputArg)
 {
     rightToLeft = YLocale::RTL();
     leftToRight = !rightToLeft;
 
     YConfig(wmapp_preferences).load(configFile);
+    if (nonempty(outputArg))
+        upath::redirectOutput(outputArg);
+    else if (nonempty(outputFile) && !(post_preferences | show_extensions))
+        upath::redirectOutput(outputFile);
 
     YXApplication::alphaBlending |= alphaBlending;
     YXApplication::synchronizeX11 |= synchronizeX11;
@@ -1765,6 +1820,7 @@ int main(int argc, char **argv) {
     bool rewrite_prefs(false);
     bool notify_parent(false);
     const char* configFile(nullptr);
+    const char* outputFile(nullptr);
     const char* displayName(nullptr);
     const char* overrideTheme(nullptr);
 
@@ -1815,6 +1871,8 @@ int main(int argc, char **argv) {
                 YXApplication::alphaBlending = true;
             else if (GetArgument(value, "d", "display", arg, argv+argc))
                 displayName = value;
+            else if (GetArgument(value, "o", "output", arg, argv+argc))
+                outputFile = value;
             else if (GetArgument(value, "s", "splash", arg, argv+argc))
                 splashFile = value;
             else if (GetLongArgument(value, "trace", arg, argv+argc))
@@ -1836,7 +1894,7 @@ int main(int argc, char **argv) {
 
     if (isEmpty(configFile))
         configFile = "preferences";
-    loadStartup(configFile);
+    loadStartup(configFile, outputFile);
 
     if (nonempty(overrideTheme)) {
         unsigned last = ACOUNT(wmapp_preferences) - 2;
@@ -1941,8 +1999,11 @@ void YWMApp::handleSMAction(WMAction message) {
         { ICEWM_ACTION_WINDOWLIST,    actionWindowList },
         { ICEWM_ACTION_RESTARTWM,     actionRestart },
         { ICEWM_ACTION_SUSPEND,       actionSuspend },
+        { ICEWM_ACTION_HIBERNATE,     actionHibernate },
         { ICEWM_ACTION_WINOPTIONS,    actionWinOptions },
         { ICEWM_ACTION_RELOADKEYS,    actionReloadKeys },
+        { ICEWM_ACTION_ICEWMBG,       actionIcewmbg },
+        { ICEWM_ACTION_REFRESH,       actionRefresh },
     };
     for (auto p : pairs)
         if (message == p.left)
