@@ -101,7 +101,7 @@ public:
     ref<YIcon> getDefaultAppIcon();
 
     bool hasCtrlAltDelete() const { return ctrlAltDelete != nullptr; }
-    CtrlAltDelete* getCtrlAltDelete();
+    CtrlAltDelete* getCtrlAltDelete() const { return ctrlAltDelete; }
     const char* getConfigFile() const { return configFile; }
     FocusModel getFocusMode() const { return focusMode; }
 
@@ -139,6 +139,7 @@ private:
     virtual Cursor getRightPointer() const { return rightPointer; }
     virtual bool handleTimer(YTimer *timer);
     virtual int handleError(XErrorEvent *xev);
+    virtual void keyboardRemap();
     void runRestart(const char *path, char *const *args);
 
     FocusModel focusMode;
@@ -152,6 +153,7 @@ private:
     void initPointers();
     void initIcons();
     void initIconSize();
+    void fixupPreferences();
 };
 
 extern YWMApp * wmapp;
