@@ -7,17 +7,18 @@
 #include "yfull.h"
 #include "wmclient.h"
 #include "prefs.h"
-
 #include "yrect.h"
 #include "wmframe.h"
 #include "wmmgr.h"
-#include "wmapp.h"
+#include "yxapp.h"
 #include "sysdep.h"
 #include "yxcontext.h"
 #include "workspaces.h"
 #include "wmminiicon.h"
 #include "intl.h"
 
+extern ref<YIcon> newClientIcon(int count, int reclen, long * elem);
+
 bool operator==(const XSizeHints& a, const XSizeHints& b) {
     long mask = PMinSize | PMaxSize | PResizeInc |
                 PAspect | PBaseSize | PWinGravity;
@@ -42,6 +43,8 @@ bool operator!=(const XSizeHints& a, const XSizeHints& b) {
     return !(a == b);
 }
 
+YArray<YFrameClient::transience> YFrameClient::fTransients;
+
 YFrameClient::YFrameClient(YWindow *parent, YFrameWindow *frame, Window win,
                            int depth, Visual *visual, Colormap colormap):
     YDndWindow(parent, win, depth, visual, colormap),
@@ -50,12 +53,15 @@ YFrameClient::YFrameClient(YWindow *parent, YFrameWindow *frame, Window win,
     fWindowRole()
 {
     fFrame = frame;
-    fBorder = 0;
+    fClientItem = nullptr;
     fProtocols = 0;
+    fBorder = 0;
     fColormap = colormap;
     fDocked = false;
     fShaped = false;
     fTimedOut = false;
+    fFixedTitle = false;
+    fIconize = true;
     fPinging = false;
     fPingTime = 0;
     fHints = nullptr;
@@ -92,16 +98,22 @@ YFrameClient::YFrameClient(YWindow *parent, YFrameWindow *frame, Window win,
             queryShape();
         }
 #endif
+        if (fClassHint.nonempty()) {
+            if (hintOptions && hintOptions->nonempty())
+                loadWindowOptions(hintOptions, true);
+            loadWindowOptions(defOptions, false);
+        }
     }
-
     clientContext.save(handle(), this);
 }
 
 YFrameClient::~YFrameClient() {
     clientContext.remove(handle());
-
+    if (fTransientFor) setTransient(None);
     if (fSizeHints) { XFree(fSizeHints); fSizeHints = nullptr; }
     if (fHints) { XFree(fHints); fHints = nullptr; }
+    if (fClientItem) { fClientItem->goodbye(); fClientItem = nullptr; }
+    if (fPinging) { fPingTimer = null; }
 }
 
 void YFrameClient::getProtocols(bool force) {
@@ -134,22 +146,28 @@ void YFrameClient::getSizeHints() {
             !XGetWMNormalHints(xapp->display(), handle(), fSizeHints, &supplied))
             fSizeHints->flags = 0;
 
-        if (notbit(fSizeHints->flags, PResizeInc)) {
+        if (notbit(fSizeHints->flags, PResizeInc) ||
+            fSizeHints->width_inc < 1 || fSizeHints->height_inc < 1) {
             fSizeHints->width_inc = fSizeHints->height_inc = 1;
         }
 
-        if (!(fSizeHints->flags & PBaseSize)) {
+        if (notbit(fSizeHints->flags, PBaseSize)) {
             if (fSizeHints->flags & PMinSize) {
                 fSizeHints->base_width = fSizeHints->min_width;
                 fSizeHints->base_height = fSizeHints->min_height;
             } else
                 fSizeHints->base_width = fSizeHints->base_height = 0;
         }
-        if (!(fSizeHints->flags & PMinSize)) {
+        if (notbit(fSizeHints->flags, PMinSize)) {
             fSizeHints->min_width = fSizeHints->base_width;
             fSizeHints->min_height = fSizeHints->base_height;
         }
-        if (!(fSizeHints->flags & PMaxSize)) {
+        if (fSizeHints->min_height < 1)
+            fSizeHints->min_height = 1;
+        if (fSizeHints->min_width < 1)
+            fSizeHints->min_width = 1;
+
+        if (notbit(fSizeHints->flags, PMaxSize)) {
             fSizeHints->max_width = 32767;
             fSizeHints->max_height = 32767;
         }
@@ -158,11 +176,6 @@ void YFrameClient::getSizeHints() {
         if (fSizeHints->max_height < fSizeHints->min_height)
             fSizeHints->max_height = 32767;
 
-        if (fSizeHints->min_height <= 0)
-            fSizeHints->min_height = 1;
-        if (fSizeHints->min_width <= 0)
-            fSizeHints->min_width = 1;
-
         if (notbit(fSizeHints->flags, PWinGravity)) {
             fSizeHints->win_gravity = NorthWestGravity;
             fSizeHints->flags |= PWinGravity;
@@ -178,12 +191,22 @@ void YFrameClient::getClassHint() {
     if (!prop.wm_class)
         return;
 
-    fClassHint.reset();
-    XGetClassHint(xapp->display(), handle(), &fClassHint);
+    fClassHint.get(handle());
+}
+
+YAction YFrameClient::action() {
+    return YAction(EAction(int(handle())));
+}
+
+void YFrameClient::setDocked(bool docked) {
+    fDocked = docked;
+    if (fDocked && fTransientFor) {
+        setTransient(None);
+    }
 }
 
 void YFrameClient::getTransient() {
-    if (!prop.wm_transient_for)
+    if (!prop.wm_transient_for || isDocked())
         return;
 
     Window newTransientFor = None;
@@ -195,19 +218,103 @@ void YFrameClient::getTransient() {
     }
 
     if (newTransientFor != fTransientFor) {
-        if (fTransientFor)
-            if (getFrame())
-                getFrame()->removeAsTransient();
-        fTransientFor = newTransientFor;
+        setTransient(newTransientFor);
         if (fTransientFor)
             if (getFrame())
                 getFrame()->addAsTransient();
     }
 }
 
+int YFrameClient::findTransient(Window handle) {
+    for (int i = 0; i < fTransients.getCount(); ++i)
+        if (handle == fTransients[i].trans)
+            return i;
+    return -1;
+}
+
+YFrameClient* YFrameClient::firstTransient() {
+    return firstTransient(handle());
+}
+
+YFrameClient* YFrameClient::firstTransient(Window handle) {
+    YFrameClient* client = nullptr;
+    for (const transience& trans : fTransients)
+        if (trans.owner == handle)
+            if ((client = clientContext.find(trans.trans)) != nullptr)
+                break;
+    return client;
+}
+
+bool YFrameClient::hasTransient() {
+    const Window h = handle();
+    for (int i = 0; i < fTransients.getCount(); ++i)
+        if (h == fTransients[i].owner)
+            return true;
+    return false;
+}
+
+YFrameClient* YFrameClient::getOwner() const {
+    return fTransientFor ? clientContext.find(fTransientFor) : nullptr;
+}
+
+YFrameClient* YFrameClient::nextTransient() {
+    YFrameClient* trans = nullptr;
+    if (fTransientFor) {
+        int i = findTransient(handle());
+        while (0 < ++i && i < fTransients.getCount()) {
+            if (fTransientFor == fTransients[i].owner) {
+                trans = clientContext.find(fTransients[i].trans);
+                if (trans)
+                    break;
+            }
+        }
+    }
+    return trans;
+}
+
+void YFrameClient::setTransient(Window owner) {
+    if (owner == None) {
+        int i = findTransient(handle());
+        if (0 <= i)
+            fTransients.remove(i);
+        fTransientFor = None;
+    }
+    else {
+        YArray<Window> cycle;
+        cycle.append(handle());
+        cycle.append(owner);
+        Window up = owner;
+        for (int i = findTransient(up); 0 <= i; i = findTransient(up)) {
+            up = fTransients[i].owner;
+            if (0 <= find(cycle, up))
+                return;
+            cycle.append(up);
+        }
+
+        int i = findTransient(handle());
+        if (0 <= i)
+            fTransients[i].owner = owner;
+        else
+            fTransients.append(transience(handle(), owner));
+        fTransientFor = owner;
+    }
+}
+
+bool YFrameClient::isTransientFor(Window owner) {
+    if (fTransientFor == None)
+        return false;
+    if (fTransientFor == owner)
+        return true;
+    YFrameClient* client = clientContext.find(fTransientFor);
+    if (client == nullptr)
+        return false;
+    return client->isTransientFor(owner);
+}
+
 void YFrameClient::constrainSize(int &w, int &h, int flags)
 {
-    if (fSizeHints && (considerSizeHintsMaximized || !getFrame()->isMaximized())) {
+    if (fSizeHints && (considerSizeHintsMaximized ||
+                       !getFrame() || !getFrame()->isMaximized())) {
         int const wMin(fSizeHints->min_width);
         int const hMin(fSizeHints->min_height);
         int const wMax(fSizeHints->max_width);
@@ -337,27 +444,32 @@ bool YFrameClient::handleTimer(YTimer* timer) {
         fPingTimer = null;
         fPinging = false;
         fTimedOut = true;
-        if (fPid == 0 && !getNetWMPid(&fPid) && fFrame && fFrame->owner()) {
-            fFrame->owner()->client()->getNetWMPid(&fPid);
+        if (fPid == 0 && !getNetWMPid(&fPid) && isTransient()) {
+            YFrameClient* owner = getOwner();
+            if (owner)
+                owner->getNetWMPid(&fPid);
         }
         if ( !destroyed()) {
             if (fFrame == nullptr) {
-                if (isDocked() && killPid() == false) {
-                    XKillClient(xapp->display(), handle());
-                }
+                forceClose();
             }
-            else if (fFrame->frameOption(YFrameWindow::foForcedClose)) {
-                if (killPid() == false) {
-                    fFrame->wmKill();
-                }
+            else if (isCloseForced()) {
+                forceClose();
             }
             else if (fPid > 0) {
                 char* res = classHint()->resource();
+                if (res && strlen(res) > 47L) {
+                    char* dot = strchr(res, '.');
+                    if (dot && inrange<long>(dot - res, 8L, 48L))
+                        *dot = '\0';
+                    else
+                        res[47] = '\0';
+                }
                 char buf[234];
                 snprintf(buf, sizeof buf,
                          _("Client %s with PID %ld fails to respond.\n"
                            "Do you wish to terminate this client?\n"),
-                         res, fPid);
+                         res ? res : "''", fPid);
                 fFrame->wmConfirmKill(buf);
                 free(res);
             }
@@ -370,6 +482,20 @@ bool YFrameClient::handleTimer(YTimer* timer) {
     return false;
 }
 
+bool YFrameClient::isCloseForced() {
+    return frameOption(YFrameWindow::foForcedClose);
+}
+
+bool YFrameClient::frameOption(int option) {
+    const WindowOption* wo = getWindowOption();
+    return wo && wo->hasOption(option);
+}
+
+bool YFrameClient::forceClose() {
+    return adopted() && (killPid() || XKillClient(xapp->display(), handle()));
+}
+
+
 bool YFrameClient::killPid() {
     return fPid > 0 && 0 == kill(fPid, SIGTERM);
 }
@@ -426,9 +552,13 @@ void YFrameClient::setFrame(YFrameWindow *newFrame) {
     fFrame = newFrame;
 }
 
+YFrameWindow* YFrameClient::obtainFrame() const {
+    return fFrame ? fFrame : dynamic_cast<YFrameWindow*>(parent()->parent());
+}
+
 void YFrameClient::setFrameState(FrameState state) {
     if (state == WithdrawnState) {
-        if (manager->wmState() != YWindowManager::wmSHUTDOWN) {
+        if (manager->notShutting()) {
             MSG(("deleting window properties id=%lX", handle()));
             Atom atoms[] = {
                 _XA_NET_FRAME_EXTENTS, _XA_NET_WM_ALLOWED_ACTIONS,
@@ -443,7 +573,8 @@ void YFrameClient::setFrameState(FrameState state) {
         }
     }
     else if (state != fSavedFrameState) {
-        Atom iconic = (state == IconicState && getFrame()->isMinimized()
+        Atom iconic = (state == IconicState && getFrame()
+                       && getFrame()->isMinimized()
                        && getFrame()->getMiniIcon())
                     ? getFrame()->getMiniIcon()->iconWindow() : None;
         Atom arg[2] = { Atom(state), iconic };
@@ -476,23 +607,16 @@ void YFrameClient::handleUnmap(const XUnmapEvent &unmap) {
     MSG(("UnmapWindow"));
     xapp->ignorable = handle();
 
-    bool unmanage = true;
-    bool destroy = false;
-    do {
-        XEvent ev;
-        if (XCheckTypedWindowEvent(xapp->display(), unmap.window,
-                                   DestroyNotify, &ev)) {
-            YWindow::handleDestroyWindow(ev.xdestroywindow);
-            manager->destroyedClient(unmap.window);
-            unmanage = false;
-        }
-        else {
-            destroy = (adopted() && destroyed() == false && testDestroyed());
-        }
-    } while (unmanage && destroy);
-    if (unmanage && isDocked() == false) {
-        manager->unmanageClient(this);
+    XEvent event;
+    event.type = 0;
+    while (XCheckTypedWindowEvent(xapp->display(), unmap.window,
+                                  DestroyNotify, &event) == False
+        && adopted() && destroyed() == false && testDestroyed()) {
     }
+    if (event.type == DestroyNotify)
+        YWindow::handleDestroyWindow(event.xdestroywindow);
+    if (event.type == DestroyNotify || isDocked() == false)
+        manager->unmanageClient(this);
 }
 
 void YFrameClient::handleProperty(const XPropertyEvent &property) {
@@ -514,15 +638,20 @@ void YFrameClient::handleProperty(const XPropertyEvent &property) {
     case XA_WM_CLASS:
         prop.wm_class = new_prop;
         if (prop.wm_class) {
-            ClassHint old(fClassHint);
+            ClassHint previous(fClassHint);
             getClassHint();
-            if (fClassHint.nonempty() && fClassHint != old) {
-                YFrameWindow* frame = getFrame();
-                if (frame){
-                    frame->getFrameHints();
+            if (fClassHint.nonempty() && fClassHint != previous) {
+                int old = fWindowOption ? fWindowOption->layer : WinLayerInvalid;
+                fWindowOption = null;
+                if (fFrame) {
+                    fFrame->getFrameHints();
+                    if (fWindowOption && validLayer(fWindowOption->layer))
+                        fFrame->setRequestedLayer(fWindowOption->layer);
+                    else if (validLayer(old))
+                        fFrame->setRequestedLayer(WinLayerNormal);
                     if (taskBarTaskGrouping) {
-                        frame->removeAppStatus();
-                        frame->updateAppStatus();
+                        fFrame->removeAppStatus();
+                        fFrame->updateAppStatus();
                     }
                 }
             }
@@ -537,8 +666,7 @@ void YFrameClient::handleProperty(const XPropertyEvent &property) {
             bool oldUrge = urgencyHint();
             getWMHints();
             if (oldPix != iconPixmapHint() || oldMask != iconMaskHint()) {
-                if (getFrame())
-                    getFrame()->updateIcon();
+                refreshIcon();
             }
             if (oldUrge != urgencyHint()) {
                 if (getFrame())
@@ -575,13 +703,13 @@ void YFrameClient::handleProperty(const XPropertyEvent &property) {
             prop.wm_state = new_prop;
         } else if (property.atom == _XA_KWM_WIN_ICON) {
             if (new_prop) prop.kwm_win_icon = true;
-            if (getFrame() && !prop.net_wm_icon && !prop.win_icons)
-                getFrame()->updateIcon();
+            if ( !prop.net_wm_icon && !prop.win_icons)
+                refreshIcon();
             prop.kwm_win_icon = new_prop;
         } else if (property.atom == _XA_WIN_ICONS) {
             if (new_prop) prop.win_icons = true;
-            if (getFrame() && !prop.net_wm_icon)
-                getFrame()->updateIcon();
+            if ( !prop.net_wm_icon)
+                refreshIcon();
             prop.win_icons = new_prop;
         } else if (property.atom == _XA_NET_WM_NAME) {
             if (new_prop) prop.net_wm_name = true;
@@ -626,8 +754,7 @@ void YFrameClient::handleProperty(const XPropertyEvent &property) {
         } else if (property.atom == _XA_NET_WM_ICON) {
             MSG(("change: net wm icon"));
             if (new_prop) prop.net_wm_icon = true;
-            if (getFrame())
-                getFrame()->updateIcon();
+            refreshIcon();
             prop.net_wm_icon = new_prop;
         } else if (property.atom == _XA_WIN_TRAY) {
             prop.win_tray = new_prop;
@@ -683,7 +810,7 @@ void YFrameClient::handleDestroyWindow(const XDestroyWindowEvent &destroyWindow)
     YWindow::handleDestroyWindow(destroyWindow);
 
     if (destroyed())
-        manager->destroyedClient(destroyWindow.window);
+        manager->unmanageClient(this);
 }
 
 #ifdef CONFIG_SHAPE
@@ -700,7 +827,8 @@ void YFrameClient::handleShapeNotify(const XShapeEvent &shape) {
                 getFrame()->setShape();
             if (fShaped && !newShaped) {
                 fShaped = newShaped;
-                getFrame()->updateMwmHints(fSizeHints);
+                if (getFrame())
+                    getFrame()->updateMwmHints(fSizeHints);
             }
         }
     }
@@ -708,17 +836,12 @@ void YFrameClient::handleShapeNotify(const XShapeEvent &shape) {
 #endif
 
 void YFrameClient::setWindowTitle(const char *title) {
-    if (fWindowTitle != title) {
+    if (fWindowTitle != title && fFixedTitle == false) {
         fWindowTitle = title;
         if (title) {
-            XChangeProperty(xapp->display(), handle(),
-                    _XA_NET_WM_VISIBLE_NAME, _XA_UTF8_STRING,
-                    8, PropModeReplace,
-                    (const unsigned char *) title,
-                    strlen(title));
+            setProperty(_XA_NET_WM_VISIBLE_NAME, _XA_UTF8_STRING, title);
         } else {
-            XDeleteProperty(xapp->display(), handle(),
-                    _XA_NET_WM_VISIBLE_NAME);
+            deleteProperty(_XA_NET_WM_VISIBLE_NAME);
         }
         if (getFrame())
             getFrame()->updateTitle();
@@ -726,17 +849,12 @@ void YFrameClient::setWindowTitle(const char *title) {
 }
 
 void YFrameClient::setIconTitle(const char *title) {
-    if (fIconTitle != title) {
+    if (fIconTitle != title && fFixedTitle == false) {
         fIconTitle = title;
         if (title) {
-            XChangeProperty(xapp->display(), handle(),
-                    _XA_NET_WM_VISIBLE_ICON_NAME, _XA_UTF8_STRING,
-                    8, PropModeReplace,
-                    (const unsigned char *) title,
-                    strlen(title));
+            setProperty(_XA_NET_WM_VISIBLE_ICON_NAME, _XA_UTF8_STRING, title);
         } else {
-            XDeleteProperty(xapp->display(), handle(),
-                    _XA_NET_WM_VISIBLE_ICON_NAME);
+            deleteProperty(_XA_NET_WM_VISIBLE_ICON_NAME);
         }
         if (getFrame())
             getFrame()->updateIconTitle();
@@ -826,10 +944,14 @@ void YFrameClient::handleClientMessage(const XClientMessageEvent &message) {
             }
         }
     } else if (message.message_type == _XA_NET_ACTIVE_WINDOW) {
-        YFrameWindow* f = getFrame();
-        if (f && !f->ignoreActivation()) {
-            f->activate();
-            f->wmRaise();
+        YFrameWindow* f = obtainFrame();
+        if (f && !frameOption(YFrameWindow::foIgnoreActivationMessages)) {
+            if (f->client() != this)
+                f->selectTab(this);
+            if (f != manager->getFocus())
+                f->activate();
+            if (f->canRaise())
+                f->wmRaise();
         }
     } else if (message.message_type == _XA_NET_CLOSE_WINDOW) {
         actionPerformed(actionClose);
@@ -881,7 +1003,7 @@ void YFrameClient::handleClientMessage(const XClientMessageEvent &message) {
             setWorkspaceHint(message.data.l[0]);
     } else if (message.message_type == _XA_WIN_LAYER) {
         long layer = message.data.l[0];
-        if (inrange(layer, WinLayerDesktop, WinLayerAboveAll)) {
+        if (validLayer(layer)) {
             if (getFrame())
                 getFrame()->actionPerformed(layerActionSet[layer]);
             else
@@ -1025,10 +1147,7 @@ void YFrameClient::netStateRequest(int action, int mask) {
 }
 
 void YFrameClient::actionPerformed(YAction action) {
-    if (getFrame()) {
-        getFrame()->actionPerformed(action, 0U);
-    }
-    else if (isDocked()) {
+    if (isDocked()) {
         if (action == actionClose) {
             Window icon = iconWindowHint();
             sendDelete();
@@ -1040,6 +1159,66 @@ void YFrameClient::actionPerformed(YAction action) {
             xapp->sync();
         }
     }
+    else if (action == actionClose) {
+        bool confirm = false;
+        YFrameWindow* frame = obtainFrame();
+        if (frame)
+            frame->wmCloseClient(this, &confirm);
+    }
+    else if (action == actionKill) {
+        forceClose();
+    }
+    else if (action == actionUntab) {
+        YFrameWindow* frame = obtainFrame();
+        if (frame)
+            frame->untab(this);
+    }
+    else if (getFrame()) {
+        getFrame()->actionPerformed(action, 0U);
+    }
+}
+
+bool YFrameClient::activateOnMap() {
+    bool yes = true;
+    if (getFrame())
+        yes &= getFrame()->isMapped() && getFrame()->visibleNow();
+    const WindowOption* wo = getWindowOption();
+    if (wo && yes) {
+        if (wo->hasOption(YFrameWindow::foDoNotFocus))
+            yes = false;
+        if ( !wo->hasOption(YFrameWindow::foIgnoreNoFocusHint)
+            && wmHint(InputHint) && !(fHints->input & True))
+            yes = false;
+        if (wo->hasOption(YFrameWindow::foDoNotFocus))
+            yes = false;
+        if (wo->hasOption(YFrameWindow::foNoFocusOnMap))
+            yes = false;
+    }
+    if (yes && getOwner()) {
+        YFrameClient* owner = getOwner();
+        while (owner && owner->getFrame() && !owner->getFrame()->focused())
+            owner = owner->getOwner();
+        bool focus = (owner != nullptr);
+        if (focus == false) {
+            for (int i = 0; i < fTransients.getCount(); ++i) {
+                if (fTransients[i].owner == fTransientFor) {
+                    Window window = fTransients[i].trans;
+                    YFrameClient* trans = clientContext.find(window);
+                    if (trans && trans->getFrame() &&
+                        trans->getFrame()->focused()) {
+                        focus = true;
+                        break;
+                    }
+                }
+            }
+        }
+        if (focus ? !focusOnMapTransientActive : !focusOnMapTransient)
+            yes = false;
+    }
+    else if (yes) {
+        yes = focusOnMap;
+    }
+    return yes;
 }
 
 void YFrameClient::getNameHint() {
@@ -1239,6 +1418,145 @@ long YFrameClient::mwmDecors() {
     return decors;
 }
 
+void YFrameClient::refreshIcon() {
+    if (fIconize == false) {
+        fIconize = true;
+    }
+    if (fFrame) {
+        fFrame->updateIcon();
+    }
+}
+
+ref<YIcon> YFrameClient::getIcon() {
+    if (fIconize) {
+        const WindowOption* wo = getWindowOption();
+        if (wo && nonempty(wo->icon)) {
+            if (fIcon == null)
+                fIcon = YIcon::getIcon(wo->icon);
+            if (fIcon != null)
+                fIconize = false;
+        }
+        if (fIconize) {
+            fIconize = false;
+            obtainIcon();
+        }
+    }
+    return fIcon;
+}
+
+void YFrameClient::obtainIcon() {
+    long count;
+    long* elem;
+    Pixmap* pixmap;
+    Atom type;
+
+    ref<YIcon> oldIcon = fIcon;
+
+    if (getNetWMIcon(&count, &elem)) {
+        ref<YImage> icons[3], largestIcon;
+        const long sizes[3] = {
+            long(YIcon::smallSize()),
+            long(YIcon::largeSize()),
+            long(YIcon::hugeSize())
+        };
+        long* largestOffset = nullptr;
+        long largestSize = 0;
+
+        // Find icons that match Small-/Large-/HugeIconSize and search
+        // for the largest icon from NET_WM_ICON set.
+        for (long *e = elem;
+             e + 2 < elem + count && e[0] > 0 && e[1] > 0;
+             e += 2 + e[0] * e[1]) {
+            long w = e[0], h = e[1], *d = e + 2;
+            if (w == h && d + w*h <= elem + count) {
+                // Maybe huge=large=small, so examine all sizes[].
+                for (int i = 0; i < 3; i++) {
+                    if (w == sizes[i] && icons[i] == null) {
+                        if (i >= 1 && sizes[i - 1] == sizes[i]) {
+                            icons[i] = icons[i - 1];
+                        } else {
+                            icons[i] = YImage::createFromIconProperty(d, w, h);
+                            if (w > largestSize) {
+                                largestOffset = d;
+                                largestSize = w;
+                                largestIcon = icons[i];
+                            }
+                        }
+                    }
+                }
+                if ((w > largestSize && largestSize < sizes[2]) ||
+                    (w > sizes[2] && w < largestSize))
+                {
+                    largestOffset = d;
+                    largestSize = w;
+                }
+            }
+        }
+
+        // Create missing icons by scaling the largest icon.
+        for (int i = 0; i < 3; i++) {
+            if (icons[i] == null) {
+                // create the largest icon
+                if (largestIcon == null && largestOffset && largestSize) {
+                    largestIcon =
+                        YImage::createFromIconProperty(largestOffset,
+                                                       largestSize,
+                                                       largestSize);
+                }
+                if (largestIcon != null) {
+                    icons[i] = largestIcon->scale(sizes[i], sizes[i]);
+                }
+            }
+        }
+        fIcon.init(new YIcon(icons[0], icons[1], icons[2]));
+        XFree(elem);
+    }
+    else if (getWinIcons(&type, &count, &elem)) {
+        if (type == _XA_WIN_ICONS)
+            fIcon = newClientIcon(elem[0], elem[1], elem + 2);
+        else // compatibility
+            fIcon = newClientIcon(count/2, 2, elem);
+        XFree(elem);
+    }
+    else if (getKwmIcon(&count, &pixmap) && count == 2) {
+        long pix[4] = {
+            long(pixmap[0]),
+            long(pixmap[1]),
+            long(iconPixmapHint()),
+            long(iconMaskHint()),
+        };
+        XFree(pixmap);
+        fIcon = newClientIcon(1 + (pix[2] != None), 2, pix);
+    }
+    else if (iconPixmapHint()) {
+        long pix[2] = {
+            long(iconPixmapHint()),
+            long(iconMaskHint()),
+        };
+        fIcon = newClientIcon(1, 2, pix);
+    }
+
+    if (fIcon == null) {
+        const char* name = classHint()->res_name;
+        if (nonempty(name)) {
+            fIcon = YIcon::getIcon(name);
+        }
+    }
+    if (fIcon == null && adopted() == false) {
+        fIcon = YIcon::getIcon("icewm");
+    }
+
+    if (fIcon != null) {
+        if (fIcon->small() == null && fIcon->large() == null) {
+            fIcon = null;
+        }
+    }
+
+    if (fIcon == null) {
+        fIcon = oldIcon;
+    }
+}
+
 bool YFrameClient::getKwmIcon(long* count, Pixmap** pixmap) {
     *count = 0;
     *pixmap = None;
@@ -1307,8 +1625,8 @@ bool YFrameClient::getLayerHint(int* layer) {
         return false;
 
     YProperty prop(this, _XA_WIN_LAYER, F32, 1, XA_CARDINAL);
-    if (prop && inrange(*prop, 0L, WinLayerCount - 1L)) {
-        *layer = *prop;
+    if (prop && validLayer(*prop)) {
+        *layer = int(*prop);
         return true;
     }
     return false;
@@ -1330,14 +1648,13 @@ bool YFrameClient::getWinTrayHint(int* tray_opt) {
     return false;
 }
 
-void YFrameClient::setStateHint() {
-    int state = getFrame()->getState();
-    MSG(("set state 0x%8X, saved 0x%8X, win 0x%lx",
-          state, fWinStateHint, handle()));
-
+void YFrameClient::setStateHint(int state) {
     if (((fWinStateHint ^ state) & WIN_STATE_NET) == 0 || destroyed()) {
         return;
     } else {
+        MSG(("0x%07lx: set state 0x%8X, saved 0x%8X", handle(),
+              state & WIN_STATE_NET, fWinStateHint == InvalidFrameState ?
+              InvalidFrameState : fWinStateHint & WIN_STATE_NET));
         fWinStateHint = state;
     }
 
@@ -1382,7 +1699,7 @@ bool YFrameClient::getNetWMStateHint(int* mask, int* state) {
     for (Atom atom : prop) {
         flags |= getMask(atom);
     }
-    if (manager->wmState() != YWindowManager::wmSTARTUP) {
+    if (manager->isStartup() == false) {
         flags &= ~WinStateFocused;
     }
     *mask = flags;
@@ -1646,6 +1963,41 @@ void YFrameClient::handleGravityNotify(const XGravityEvent &gravity) {
     }
 }
 
+const WindowOption* YFrameClient::getWindowOption() {
+    if (fWindowOption && fWindowOption->outdated()) {
+        fWindowOption = null;
+        loadWindowOptions(defOptions, false);
+    }
+    return fWindowOption._ptr();
+}
+
+void YFrameClient::loadWindowOptions(WindowOptions* list, bool remove) {
+    const char* klass = fClassHint.res_class;
+    const char* cname = fClassHint.res_name;
+    WindowOption& wo = *fWindowOption;
+
+    if (nonempty(klass)) {
+        if (nonempty(cname)) {
+            mstring klass_instance(klass, ".", cname);
+            list->mergeWindowOption(wo, klass_instance, remove);
+
+            mstring name_klass(cname, ".", klass);
+            list->mergeWindowOption(wo, name_klass, remove);
+        }
+        list->mergeWindowOption(wo, klass, remove);
+    }
+    if (nonempty(cname)) {
+        if (fWindowRole != null) {
+            mstring name_role(cname, ".", fWindowRole);
+            list->mergeWindowOption(wo, name_role, remove);
+        }
+        list->mergeWindowOption(wo, cname, remove);
+    }
+    if (fWindowRole != null)
+        list->mergeWindowOption(wo, fWindowRole, remove);
+    list->mergeWindowOption(wo, null, remove);
+}
+
 bool ClassHint::match(const char* resource) const {
     if (isEmpty(resource))
         return false;
@@ -1669,4 +2021,10 @@ char* ClassHint::resource() const {
     return str == "." ? nullptr : strdup(str);
 }
 
+bool ClassHint::get(Window win) {
+    reset();
+    YProperty::fRequest = XA_WM_CLASS;
+    return XGetClassHint(xapp->display(), win, this);
+}
+
 // vim: set sw=4 ts=4 et:
