@@ -41,7 +41,6 @@ private:
 };
 
 class DProgram: public DObject {
-    friend class MenuProgSwitchItems;
 public:
     virtual ~DProgram();
 
@@ -55,9 +54,12 @@ public:
         ref<YIcon> icon,
         const bool restart,
         const char *wmclass,
-        upath exe,
+        const char *exe,
         YStringArray &args);
 
+    const char* cmd() const { return fCmd; }
+    YStringArray& args() { return fArgs; }
+
 protected:
     DProgram(
         IApp *app,
@@ -66,14 +68,14 @@ protected:
         ref<YIcon> icon,
         const bool restart,
         const char *wmclass,
-        upath exe,
+        const char *exe,
         YStringArray &args);
 
 private:
     const bool fRestart;
     const char *fRes;
     long fPid;
-    upath fCmd;
+    const char* fCmd;
     YStringArray fArgs;
     YSMListener *smActionListener;
 };
@@ -114,7 +116,7 @@ public:
         YSMListener *smActionListener,
         YActionListener *wmActionListener,
         mstring name,
-        upath command,
+        const char* command,
         YStringArray &args,
         long timeout = 60L,
         YWindow *parent = nullptr);
@@ -125,7 +127,7 @@ public:
 
 private:
     mstring fName;
-    upath fCommand;
+    const char* fCommand;
     YStringArray fArgs;
     time_t fModTime;
     long fTimeout;
@@ -162,13 +164,14 @@ private:
 };
 
 /**
- * Management item which wraps DProgram and holds the trigger key information.
+ * Management item that wraps DProgram and holds the trigger key information.
  */
 class KProgram {
 public:
     KProgram(const char *key, DProgram *prog, bool bIsDynSwitchMenuProg);
-    ~KProgram() { delete fProg; }
+    ~KProgram() { delete fProg; delete[] fDef; }
 
+    void parse();
     bool isKey(KeySym key, unsigned int mod) {
         return (key == fKey && mod == fMod);
     }
@@ -177,6 +180,7 @@ public:
     unsigned int modifiers() { return fMod; }
 
 private:
+    const char* fDef;
     KeySym fKey;
     unsigned int fMod;
     // not a program starter but custom switch menu
