@@ -336,6 +336,7 @@ void TaskBar::initApplets() {
         fApplications->setImage(taskbarStartImage);
         fApplications->setToolTip(_("Favorite Applications"));
         fApplications->setTitle("TaskBarMenu");
+        fApplications->realize();
     } else
         fApplications = nullptr;
 
@@ -361,6 +362,7 @@ void TaskBar::initApplets() {
         fWinList->setActionListener(this);
         fWinList->setToolTip(_("Window List Menu"));
         fWinList->setTitle("ShowWindowList");
+        fWinList->realize();
     } else
         fWinList = nullptr;
     if (taskBarShowShowDesktopButton) {
@@ -370,10 +372,16 @@ void TaskBar::initApplets() {
         fShowDesktop->setActionListener(wmActionListener);
         fShowDesktop->setToolTip(_("Show Desktop"));
         fShowDesktop->setTitle("ShowDesktop");
+        fShowDesktop->realize();
     }
 
+    unsigned tall = 1;
+    if (taskBarDoubleHeight == false) {
+        for (YWindow* w = firstWindow(); w; w = w->nextSibling())
+            tall = max(tall, w->height());
+    }
     fWorkspaces = taskBarShowWorkspaces
-                ? new WorkspacesPane(this)
+                ? new WorkspacesPane(this, tall)
                 : new AWorkspaces(this);
     fWorkspaces->setTitle("Workspaces");
 
@@ -413,6 +421,7 @@ void TaskBar::initApplets() {
     }
 
     if (fCollapseButton) {
+        fCollapseButton->realize();
         fCollapseButton->raise();
     }
 }
@@ -536,8 +545,9 @@ void TaskBar::updateLayout(unsigned &size_w, unsigned &size_h) {
         }
     }
 
-    unsigned w = (desktop->getScreenGeometry().width()
-                  * unsigned(taskBarWidthPercentage)) / 100U;
+    const unsigned dw = desktop->getScreenGeometry().width();
+    const unsigned tw = (dw * unsigned(taskBarWidthPercentage)) / 100U;
+    unsigned w = tw;
 
     if (taskBarAtTop) { // !!! for now
         y[1] = 0;
@@ -610,8 +620,10 @@ void TaskBar::updateLayout(unsigned &size_w, unsigned &size_h) {
     }
     if (fAddressBar) {
         int row = taskBarDoubleHeight;
-        YRect r(left[row], y[row] + 2,
-                max(1U, unsigned(right[row] - left[row])), h[row] - 4);
+        int wid = right[row] - left[row];
+        if (wid < 1)
+            wid = max(1, int(tw) - left[row]);
+        YRect r(left[row], y[row] + 2, unsigned(wid), h[row] - 4);
         if (rightToLeft) {
             r.xx = w - r.xx - r.ww;
         }
@@ -758,7 +770,10 @@ void TaskBar::updateWMHints() {
     if (fStrut != strut) {
         fStrut = strut;
         MSG(("SET NET WM STRUT"));
-        setProperty(_XA_NET_WM_STRUT, XA_CARDINAL, &strut, 4);
+        if (*strut)
+            setProperty(_XA_NET_WM_STRUT, XA_CARDINAL, &strut, 4);
+        else
+            deleteProperty(_XA_NET_WM_STRUT);
     }
 }
 
@@ -1003,6 +1018,7 @@ void TaskBar::actionPerformed(YAction action, unsigned int modifiers) {
 void TaskBar::handleCollapseButton() {
     fIsCollapsed = !fIsCollapsed;
     if (fCollapseButton) {
+        fCollapseButton->hide();
         ref<YImage> image = (leftToRight == fIsCollapsed)
                           ? taskbarExpandImage : taskbarCollapseImage;
         const char* text = (leftToRight == fIsCollapsed) ? "<" : ">";
@@ -1013,7 +1029,7 @@ void TaskBar::handleCollapseButton() {
             fCollapseButton->setText(text);
         }
         fCollapseButton->setToolTip(ttip);
-        fCollapseButton->repaint();
+        fCollapseButton->realize();
     }
 
     if (fIsCollapsed)
