@@ -43,8 +43,8 @@
 
 struct WMKey {
     KeySym key;
-    unsigned int mod;
-    const char *name;
+    unsigned mod;
+    const char* name;
     bool initial;
 
     bool eq(KeySym k, unsigned m) const { return key == k && mod == m; }
@@ -150,6 +150,7 @@ public:
     static bool findLoadConfigFile(cfoption* options, const char* name);
     static bool findLoadThemeFile(cfoption* options);
     static bool parseKey(const char* arg, KeySym* key, unsigned int* mod);
+    static KeySym parseKeySym(const char* arg);
     static size_t cfoptionSize();
 
 private:
