@@ -10,16 +10,18 @@
 
 #include "config.h"
 #include "ylocale.h"
+#include "ascii.h"
 #include "base.h"
 #include "intl.h"
 #include <string.h>
+#include <stdlib.h>
+#include <stdio.h>
+#include <wchar.h>
 
 #ifdef CONFIG_I18N
 #include <errno.h>
 #include <langinfo.h>
 #include <locale.h>
-#include <stdlib.h>
-#include <wchar.h>
 #include <assert.h>
 #include <X11/Xlib.h>
 #include <iconv.h>
@@ -34,19 +36,22 @@ public:
     iconv_t unicode() const { return toUnicode; }
     iconv_t localer() const { return toLocale; }
     const char* localeName() const { return fLocaleName; }
+    const char* codesetName() const { return fCodeset; }
+    const char* modifiers() const { return fModifiers; }
 
 private:
     void getConverters();
-    iconv_t getConverter(char const *from, char const **& to);
+    iconv_t getConverter(const char* from, const char**& to);
     const char* getCodeset();
 
     iconv_t toUnicode;
     iconv_t toLocale;
     const char* fLocaleName;
+    const char* fModifiers;
     const char* fCodeset;
 };
 
-YConverter::YConverter(char const* localeName) :
+YConverter::YConverter(const char* localeName) :
     toUnicode(invalid),
     toLocale(invalid),
     fLocaleName(setlocale(LC_ALL, localeName))
@@ -56,21 +61,17 @@ YConverter::YConverter(char const* localeName) :
                "Falling back to 'C' locale'."));
         fLocaleName = setlocale(LC_ALL, "C");
     }
-
+    fModifiers = XSetLocaleModifiers("");
     fCodeset = getCodeset();
 
-    extern bool multiByte;
-    multiByte = true;
-
-    MSG(("locale: %s, MB_CUR_MAX: %zd, "
-         "multibyte: %d, codeset: %s, endian: %c",
-         fLocaleName, MB_CUR_MAX, multiByte, fCodeset, little() ? 'l' : 'b'));
+    MSG(("locale: %s, MB_CUR_MAX: %zd, codeset: %s, endian: %c",
+         fLocaleName, MB_CUR_MAX, fCodeset, little() ? 'l' : 'b'));
 
     getConverters();
 }
 
 const char* YConverter::getCodeset() {
-    char const* codeset = nullptr;
+    const char* codeset = nullptr;
     int const codesetItems[] = {
 #ifdef CONFIG_NL_CODESETS
         CONFIG_NL_CODESETS
@@ -102,7 +103,7 @@ void YConverter::getConverters() {
 
     // #warning "this is getting way too complicated"
 
-    char const * unicodeCharsets[] = {
+    const char* unicodeCharsets[] = {
 #ifdef CONFIG_UNICODE_SET
         CONFIG_UNICODE_SET,
 #endif
@@ -115,13 +116,13 @@ void YConverter::getConverters() {
         nullptr
     };
 
-    char const* localeCharsets[] = {
+    const char* localeCharsets[] = {
         cstrJoin(fCodeset, "//TRANSLIT", nullptr),
         fCodeset,
         nullptr
     };
 
-    char const** ucs(unicodeCharsets);
+    const char** ucs(unicodeCharsets);
     toUnicode = getConverter(localeCharsets[1], ucs);
     if (toUnicode == invalid)
         die(1, _("iconv doesn't supply (sufficient) "
@@ -129,7 +130,7 @@ void YConverter::getConverters() {
 
     MSG(("toUnicode converts from %s to %s", localeCharsets[1], *ucs));
 
-    char const** lcs(localeCharsets);
+    const char** lcs(localeCharsets);
     toLocale = getConverter(*ucs, lcs);
     if (toLocale == invalid)
         die(1, _("iconv doesn't supply (sufficient) "
@@ -157,16 +158,20 @@ iconv_t YConverter::getConverter(const char* from, const char**& to) {
 
 YLocale* YLocale::instance;
 
-YLocale::YLocale(char const* localeName)
+YLocale::YLocale(const char* localeName)
     : converter(nullptr)
+    , rightToLeft(false)
+    , codesetUTF8(false)
 {
     if (instance == nullptr) {
         instance = this;
 #ifdef CONFIG_I18N
         converter = new YConverter(localeName);
+        codesetUTF8 = (0 == strncmp(converter->codesetName(), "UTF-8", 5));
 #endif
         bindtextdomain(PACKAGE, LOCDIR);
         textdomain(PACKAGE);
+        getDirection();
     }
 }
 
@@ -180,49 +185,56 @@ YLocale::~YLocale() {
 }
 
 #ifdef CONFIG_I18N
-YLChar * YLocale::localeString(YUChar const *uStr, size_t uLen, size_t &lLen) {
+char* YLocale::localeString(const wchar_t* uStr, size_t uLen, size_t &lLen) {
     PRECONDITION(instance);
     if (uStr == nullptr)
         return nullptr;
 
-    YLChar* lStr(new YLChar[uLen + 1]);
+    iconv(instance->converter->localer(), nullptr, nullptr, nullptr, nullptr);
+
+    size_t lSize = 4 * uLen;
+    char* lStr = new char[lSize + 1];
 #ifdef __NetBSD__
     const
 #endif
-    char* inbuf((char *) uStr);
-    char* outbuf((char *) lStr);
-    size_t inlen(uLen), outlen(uLen);
+    char* inbuf = (char *) uStr;
+    char* outbuf = lStr;
+    size_t inlen = uLen * sizeof(wchar_t);
+    size_t outlen = lSize;
 
     errno = 0;
     size_t count = iconv(instance->converter->localer(),
                          &inbuf, &inlen, &outbuf, &outlen);
     if (count == size_t(-1)) {
         static unsigned count, shift;
-        if (++count >= (1U << shift)) {
+        if (++count <= 2 || (count - 2) >= (1U << shift)) {
             ++shift;
-            warn("Invalid unicode string: %s", strerror(errno));
+            warn("Invalid unicode string: %s (%zd/%u)",
+                 strerror(errno), ((wchar_t*)inbuf - uStr), *inbuf);
         }
     }
 
-    *((YLChar *) outbuf) = 0;
-    lLen = ((YLChar *) outbuf) - lStr;
+    *outbuf = '\0';
+    lLen = outbuf - lStr;
 
     return lStr;
 }
 
-YUChar *YLocale::unicodeString(const YLChar* lStr, size_t const lLen,
+wchar_t* YLocale::unicodeString(const char* lStr, size_t const lLen,
                                size_t& uLen)
 {
     PRECONDITION(instance);
     if (lStr == nullptr)
         return nullptr;
 
-    YUChar* uStr(new YUChar[lLen + 1]);
+    iconv(instance->converter->unicode(), nullptr, nullptr, nullptr, nullptr);
+
+    wchar_t* uStr(new wchar_t[lLen + 1]);
 #ifdef __NetBSD__
     const
 #endif
-    char* inbuf((char *) lStr);
-    char* outbuf((char *) uStr);
+    char* inbuf(const_cast<char *>(lStr));
+    char* outbuf(reinterpret_cast<char *>(uStr));
     size_t inlen(lLen), outlen(4 * lLen);
 
     errno = 0;
@@ -236,13 +248,80 @@ YUChar *YLocale::unicodeString(const YLChar* lStr, size_t const lLen,
         }
     }
 
-    *((YUChar *) outbuf) = 0;
-    uLen = ((YUChar *) outbuf) - uStr;
+    *(reinterpret_cast<wchar_t *>(outbuf)) = 0;
+    uLen = reinterpret_cast<wchar_t *>(outbuf) - uStr;
 
     return uStr;
 }
+#else
+
+wchar_t* YLocale::wideCharString(const char* str, size_t len, size_t& out) {
+    wchar_t* text = new wchar_t[len + 1];
+    size_t count = 0;
+    mbtowc(nullptr, nullptr, size_t(0));
+    for (size_t i = 0; i < len; ) {
+        int k = mbtowc(&text[count], str + i, len - i);
+        if (k < 1) {
+            i++;
+        } else {
+            i += k;
+            count++;
+        }
+    }
+    text[count] = 0;
+    out = count;
+    return text;
+}
 #endif
 
+char* YLocale::narrowString(const wchar_t* uStr, size_t uLen, size_t& lLen) {
+    PRECONDITION(instance);
+    if (uStr == nullptr || uLen == 0) {
+        lLen = 0;
+        return nullptr;
+    }
+
+    size_t size = 4 + 3 * uLen / 2;
+    char* dest = new char[size + 1];
+    size_t done;
+
+    for (;;) {
+        const wchar_t* ptr = uStr;
+        mbstate_t state;
+        memset(&state, 0, sizeof(mbstate_t));
+        done = wcsrtombs(dest, &ptr, size, &state);
+        if (done == size_t(-1)) {
+            done = (ptr > uStr) ? ptr - uStr : 0;
+            if (done + 4 >= size) {
+                delete[] dest;
+                size = 4 + 3 * size / 2;
+                dest = new char[size + 1];
+            } else {
+                break;
+            }
+        } else {
+            break;
+        }
+    }
+
+    if (done == 0) {
+        delete[] dest;
+        dest = nullptr;
+    }
+    else if (2 * done < size && 30 < size) {
+        char* copy = new char[done + 1];
+        memcpy(copy, dest, done);
+        copy[done] = '\0';
+        delete[] dest;
+        dest = copy;
+    } else {
+        dest[done] = '\0';
+    }
+
+    lLen = done;
+    return dest;
+}
+
 const char *YLocale::getLocaleName() {
 #ifdef CONFIG_I18N
     return instance->converter->localeName();
@@ -264,4 +343,27 @@ int YLocale::getRating(const char *localeStr) {
     return i;
 }
 
+void YLocale::getDirection() {
+#ifdef CONFIG_I18N
+    using namespace ASCII;
+    const char* loc = converter ? converter->localeName() : "C";
+    if (loc && isLower(*loc) && isLower(loc[1]) && !isAlpha(loc[2])) {
+        const char rtls[][4] = {
+            "ar",   // arabic
+            "fa",   // farsi
+            "he",   // hebrew
+            "ps",   // pashto
+            "sd",   // sindhi
+            "ur",   // urdu
+        };
+        for (auto rtl : rtls) {
+            if (rtl[0] == loc[0] && rtl[1] == loc[1]) {
+                rightToLeft = true;
+                break;
+            }
+        }
+    }
+#endif
+}
+
 // vim: set sw=4 ts=4 et:
