@@ -386,9 +386,9 @@ Window YWindow::create() {
     }
     fDepth = unsigned(wa.depth);
     fVisual = wa.visual;
-    if (parent() == desktop && !(flags & (wsManager | wsOverrideRedirect))) {
+    if (parent() == desktop && !(fStyle & (wsManager | wsOverrideRedirect))) {
         Atom prot[] = { _XA_WM_DELETE_WINDOW, _XA_WM_TAKE_FOCUS };
-        const int n = 1 + hasbit(flags, wsTakeFocus);
+        const int n = 1 + hasbit(fStyle, wsTakeFocus);
         XSetWMProtocols(xapp->display(), fHandle, prot, n);
     }
 
@@ -1987,13 +1987,48 @@ int YWindow::getScreen() {
     return desktop->getScreenForRect(dx, dy, width(), height());
 }
 
+bool YDesktop::addScreen(DesktopScreenInfo& info) {
+    for (DesktopScreenInfo& xi : xiInfo) {
+        if (xi == info) {
+            xi.usage += 1;
+            if (xi.usage == 1)
+                xi.screen_number = info.screen_number;
+            return true;
+        }
+    }
+    info.usage = 1;
+    xiInfo += info;
+    return false;
+}
+
 bool YDesktop::updateXineramaInfo(unsigned& horizontal, unsigned& vertical) {
-    xiInfo.clear();
+    bool change = false;
+
+    for (DesktopScreenInfo& xi : xiInfo) {
+        xi.usage = 0;
+    }
 
 #ifdef CONFIG_XRANDR
     if (xrandr.supported && !xrrDisable) {
         XRRScreenResources *xrrsr =
             XRRGetScreenResources(xapp->display(), handle());
+        RRCrtc primaryCRT = RRCrtc(-1);
+        DesktopScreenInfo primaryInfo(0, 0, 0, 0, 0);
+
+        if (nonempty(xineramaPrimaryScreenName)) {
+            const char* name = xineramaPrimaryScreenName;
+            for (int i = 0; xrrsr && i < xrrsr->noutput; i++) {
+                XRROutputInfo *oinfo = XRRGetOutputInfo(xapp->display(), xrrsr,
+                                                        xrrsr->outputs[i]);
+                MSG(("output: %s -> %lu", oinfo->name, oinfo->crtc));
+                if (oinfo->name && strcmp(oinfo->name, name) == 0) {
+                    primaryCRT = oinfo->crtc;
+                    break;
+                }
+                XRRFreeOutputInfo(oinfo);
+            }
+        }
+
         for (int i = 0; xrrsr && i < xrrsr->ncrtc; i++) {
             XRRCrtcInfo *ci = XRRGetCrtcInfo(xapp->display(), xrrsr,
                                              xrrsr->crtcs[i]);
@@ -2002,43 +2037,48 @@ bool YDesktop::updateXineramaInfo(unsigned& horizontal, unsigned& vertical) {
             if (ci->width && ci->height) {
                 DesktopScreenInfo si(int(xrrsr->crtcs[i]),
                                      ci->x, ci->y, ci->width, ci->height);
-                xiInfo.append(si);
+                change |= addScreen(si);
+                if (primaryCRT == xrrsr->crtcs[i])
+                    primaryInfo = si;
             }
             XRRFreeCrtcInfo(ci);
         }
-
-        if (xineramaPrimaryScreenName) {
-            const char* name = xineramaPrimaryScreenName;
-            MSG(("xinerama primary screen name: %s", name));
-            for (int o = 0; xrrsr && o < xrrsr->noutput; o++) {
-                XRROutputInfo *oinfo = XRRGetOutputInfo(xapp->display(), xrrsr,
-                                                        xrrsr->outputs[o]);
-                MSG(("output: %s -> %lu", oinfo->name, oinfo->crtc));
-                if (oinfo->name && strcmp(oinfo->name, name) == 0) {
-                    for (int k = 0; k < xiInfo.getCount(); k++) {
-                         if (xiInfo[k].screen_number == int(oinfo->crtc)) {
-                             xineramaPrimaryScreen = o;
-                             MSG(("xinerama primary screen: %s -> %d",
-                                         oinfo->name, o));
-                         }
-                    }
-                }
-                XRRFreeOutputInfo(oinfo);
+        for (int i = getScreenCount(); --i >= 0; ) {
+            if (xiInfo[i].usage == 0) {
+                xiInfo.remove(i);
+                change = true;
             }
         }
+
+        if (primaryInfo.width && primaryInfo.height) {
+            for (int i = 0; i < xiInfo.getCount(); ++i) {
+                if (xiInfo[i] == primaryInfo) {
+                     xineramaPrimaryScreen = i;
+                     MSG(("xinerama primary screen: %s -> %d",
+                           xineramaPrimaryScreenName, i));
+                     break;
+                }
+            }
+        }
+
         XRRFreeScreenResources(xrrsr);
     }
 #endif
 
 #ifdef XINERAMA
-    if (xiInfo.getCount() < 2 && xinerama.supported) {
+    for (int i = xiInfo.getCount(); --i >= 0; ) {
+        if (xiInfo[i].usage == 0) {
+            xiInfo.remove(i);
+            change = true;
+        }
+    }
+    if (xiInfo.isEmpty() && xinerama.supported) {
         // use xinerama if no XRANDR screens (nvidia hack)
         int count = 0;
         xsmart<XineramaScreenInfo> screens(
                XineramaQueryScreens(xapp->display(), &count));
         MSG(("xinerama: heads=%d", count));
-        if (screens && count > xiInfo.getCount()) {
-            xiInfo.clear();
+        if (screens) {
             for (int i = 0; i < count; i++) {
                 const XineramaScreenInfo& xine(screens[i]);
                 MSG(("xinerama: %d +%d+%d %dx%d", xine.screen_number,
@@ -2046,7 +2086,7 @@ bool YDesktop::updateXineramaInfo(unsigned& horizontal, unsigned& vertical) {
                 DesktopScreenInfo si(i, xine.x_org, xine.y_org,
                                      unsigned(xine.width),
                                      unsigned(xine.height));
-                xiInfo.append(si);
+                change |= addScreen(si);
             }
         }
     }
@@ -2056,7 +2096,7 @@ bool YDesktop::updateXineramaInfo(unsigned& horizontal, unsigned& vertical) {
         DesktopScreenInfo si(0, 0, 0,
                              unsigned(xapp->displayWidth()),
                              unsigned(xapp->displayHeight()));
-        xiInfo.append(si);
+        change |= addScreen(si);
     }
 
     unsigned w = 0;
@@ -2071,7 +2111,7 @@ bool YDesktop::updateXineramaInfo(unsigned& horizontal, unsigned& vertical) {
     swap(w, horizontal);
     swap(h, vertical);
     MSG(("desktop screen area: %u %u -> %u %u", w, h, horizontal, vertical));
-    return w != horizontal || h != vertical;
+    return change || w != horizontal || h != vertical;
 }
 
 const DesktopScreenInfo& YDesktop::getScreenInfo(int screen_no) {
