@@ -21,9 +21,10 @@ struct DesktopScreenInfo {
     int y_org;
     unsigned width;
     unsigned height;
+    unsigned usage;
 
     DesktopScreenInfo(int i, int x, int y, unsigned w, unsigned h) :
-        screen_number(i), x_org(x), y_org(y), width(w), height(h)
+        screen_number(i), x_org(x), y_org(y), width(w), height(h), usage(0)
     { }
     operator YRect() const {
         return YRect(x_org, y_org, width, height);
@@ -44,6 +45,14 @@ struct DesktopScreenInfo {
         return (1L + horizontalCoverage(x, w))
              * (1L + verticalCoverage(y, h));
     }
+    bool operator==(const DesktopScreenInfo& o) {
+        return x_org == o.x_org && y_org == o.y_org
+            && width == o.width && height == o.height;
+    }
+    bool operator!=(const DesktopScreenInfo& o) {
+        return x_org != o.x_org || y_org != o.y_org
+            || width != o.width || height != o.height;
+    }
 };
 
 class AToolTip {
@@ -166,6 +175,7 @@ public:
     Window handle() { return (flags & wfCreated) ? fHandle : create(); }
     YWindow *parent() const { return fParent; }
     YWindow *window() { return this; }
+    YWindow *nextSibling() { return nextWindow(); }
 
     void paintExpose(int ex, int ey, int ew, int eh);
 
@@ -371,7 +381,7 @@ private:
 
 class YDesktop: public YWindow {
 public:
-    YDesktop(YWindow *aParent = nullptr, Window win = 0);
+    YDesktop(YWindow *aParent, Window win);
     virtual ~YDesktop();
 
     bool updateXineramaInfo(unsigned& horizontal, unsigned& vertical);
@@ -390,6 +400,9 @@ public:
 
 protected:
     YArray<DesktopScreenInfo> xiInfo;
+
+private:
+    bool addScreen(DesktopScreenInfo& info);
 };
 
 extern YDesktop *desktop;
