@@ -6,6 +6,7 @@
 #include "MwmUtil.h"
 #include "ypointer.h"
 #include "yxcontext.h"
+#include "yconfig.h"
 #include "guievent.h"
 #include "intl.h"
 #undef override
@@ -56,6 +57,7 @@ Atom _XA_ICEWM_ACTION;
 Atom _XA_ICEWM_GUIEVENT;
 Atom _XA_ICEWM_HINT;
 Atom _XA_ICEWM_FONT_PATH;
+Atom _XA_ICEWM_TABS;
 Atom _XA_ICEWMBG_IMAGE;
 Atom _XA_XROOTPMAP_ID;
 Atom _XA_XROOTCOLOR_PIXEL;
@@ -306,6 +308,7 @@ YAtomName YXApplication::atom_info[] = {
     { &_XA_ICEWM_GUIEVENT                   , XA_GUI_EVENT_NAME },
     { &_XA_ICEWM_HINT                       , "_ICEWM_WINOPTHINT" },
     { &_XA_ICEWM_FONT_PATH                  , "ICEWM_FONT_PATH" },
+    { &_XA_ICEWM_TABS                       , "_ICEWM_TABS" },
     { &_XA_ICEWMBG_IMAGE                    , "_ICEWMBG_IMAGE" },
     { &_XA_XROOTPMAP_ID                     , "_XROOTPMAP_ID" },
     { &_XA_XROOTCOLOR_PIXEL                 , "_XROOTCOLOR_PIXEL" },
@@ -598,6 +601,11 @@ void YXApplication::initModifiers() {
          AltMask, MetaMask, SuperMask, HyperMask, WinMask, ModeSwitchMask,
          NumLockMask, ScrollLockMask));
 
+    fKeycodeMin = fKeycodeMax = fKeysymsPer = 0;
+    if (fKeycodeMap) {
+        XFree(fKeycodeMap);
+        fKeycodeMap = nullptr;
+    }
 }
 
 bool YXApplication::hasControlAlt(unsigned state) const {
@@ -808,6 +816,7 @@ bool YXApplication::filterEvent(const XEvent &xev) {
         }
 
         initModifiers();
+        keyboardRemap();
 
         desktop->grabKeys();
         desktop->kbLayout();
@@ -1072,6 +1081,10 @@ YXApplication::YXApplication(int *argc, char ***argv, const char *displayName):
     xfd(this),
     fXGrabWindow(nullptr),
     fGrabWindow(nullptr),
+    fKeycodeMap(nullptr),
+    fKeycodeMin(0),
+    fKeycodeMax(0),
+    fKeysymsPer(0),
     fGrabTree(false),
     fGrabMouse(false),
     fReplayEvent(false)
@@ -1126,6 +1139,8 @@ void YXApplication::initExtensions(Display* dpy) {
 YXApplication::~YXApplication() {
     if (fColormap32)
         XFreeColormap(display(), fColormap32);
+    if (fKeycodeMap)
+        XFree(fKeycodeMap);
 
     xfd.unregisterPoll();
     XCloseDisplay(display());
@@ -1232,6 +1247,9 @@ void YXApplication::handleWindowEvent(Window xwindow, XEvent &xev) {
                     xev.xmaprequest.parent));
                 desktop->handleEvent(xev);
             }
+            else if (windowExists(xev.xmaprequest.window)) {
+                desktop->handleEvent(xev);
+            }
         } else if (xev.type == ConfigureRequest) {
             if (xev.xconfigurerequest.window != ignorable) {
                 TLOG(("APP BUG? configureRequest for window %lX "
@@ -1240,6 +1258,9 @@ void YXApplication::handleWindowEvent(Window xwindow, XEvent &xev) {
                     xev.xconfigurerequest.parent));
                 desktop->handleEvent(xev);
             }
+            else if (windowExists(xev.xconfigurerequest.window)) {
+                desktop->handleEvent(xev);
+            }
         }
         else if (xev.type == ClientMessage && desktop) {
             Atom mesg = xev.xclient.message_type;
@@ -1274,11 +1295,17 @@ int YXApplication::errorHandler(Display* display, XErrorEvent* xev) {
     if (rc == Success)
         return rc;
 
-    XDBG if (xev->resourceid != ignorable) {
+#if defined(DEBUG) || defined(PRECON)
+    if (xev->resourceid != ignorable) {
         char message[80], req[80], number[80];
 
-        snprintf(number, sizeof number, "%d", xev->request_code);
-        XGetErrorDatabaseText(display, "XRequest", number, "", req, sizeof req);
+        if (xev->request_code == X_GetProperty)
+            snprintf(req, sizeof req, "X_GetProperty(%s)",
+                     atomName(YProperty::fRequest));
+        else {
+            snprintf(number, sizeof number, "%d", xev->request_code);
+            XGetErrorDatabaseText(display, "XRequest", number, "", req, 80);
+        }
         if (req[0] == 0)
             snprintf(req, sizeof req, "[request_code=%d]", xev->request_code);
 
@@ -1290,7 +1317,6 @@ int YXApplication::errorHandler(Display* display, XErrorEvent* xev) {
              long(NextRequest(display)) - long(xev->serial),
              long(LastKnownRequestProcessed(display)) - long(xev->serial));
 
-#if defined(DEBUG) || defined(PRECON)
         if (xapp->synchronized()) {
             switch (xev->request_code) {
                 case X_GetWindowAttributes:
@@ -1307,8 +1333,8 @@ int YXApplication::errorHandler(Display* display, XErrorEvent* xev) {
         else if (ONCE) {
             TLOG(("unsynchronized"));
         }
-#endif
     }
+#endif
 
     if (rc == BadImplementation)
         xapp->exit(rc);
@@ -1336,6 +1362,59 @@ bool YXApplication::children(Window win, Window** data, unsigned* num) const {
     return XQueryTree(display(), win, &rootw, &paren, data, num);
 }
 
+bool YXApplication::queryMask(Window w, unsigned* mask) {
+    Window root, child;
+    int rx, ry, wx, wy;
+    return XQueryPointer(display(), w, &root, &child, &rx, &ry, &wx, &wy, mask);
+}
+
+void YXApplication::queryMouse(int* x, int* y) {
+    Window root, child;
+    int wx, wy;
+    unsigned mask;
+    if (XQueryPointer(display(), desktop->handle(),
+                      &root, &child, x, y, &wx, &wy, &mask) == False)
+        *x = *y = 0;
+}
+
+bool YXApplication::parseKey(const char* arg, KeySym* key, unsigned* mod) {
+    bool yes = YConfig::parseKey(arg, key, mod);
+    if (yes)
+        unshift(key, mod);
+    return yes;
+}
+
+void YXApplication::unshift(KeySym* ksym, unsigned* mod) {
+    const unsigned key = unsigned(*ksym);
+    if ((' ' < key && key < 'a') || ('z' < key && key <= 0xff)) {
+        if (fKeycodeMap == nullptr) {
+            XDisplayKeycodes(xapp->display(), &fKeycodeMin, &fKeycodeMax);
+            fKeycodeMap = XGetKeyboardMapping(xapp->display(), fKeycodeMin,
+                                              fKeycodeMax - fKeycodeMin + 1,
+                                              &fKeysymsPer);
+        }
+        if (fKeycodeMap && 1 < fKeycodeMax && 1 < fKeysymsPer) {
+            const int lo = fKeycodeMin;
+            int un = 0, sh = 0;
+            for (int i = lo; i <= fKeycodeMax; ++i) {
+                if (fKeycodeMap[(i - lo) * fKeysymsPer] == key && un == 0)
+                    un = i;
+                if (fKeycodeMap[(i - lo) * fKeysymsPer + 1] == key && sh == 0)
+                    sh = i;
+            }
+            if (sh && un == 0 && fKeycodeMap[(sh - lo) * fKeysymsPer]) {
+                *ksym = fKeycodeMap[(sh - lo) * fKeysymsPer];
+                *mod |= ShiftMask;
+            }
+        }
+    }
+}
+
+bool YXApplication::windowExists(Window handle) const {
+    XWindowAttributes attributes;
+    return XGetWindowAttributes(display(), handle, &attributes);
+}
+
 void YXPoll::notifyRead() {
     owner()->handleXEvents();
 }
@@ -1395,9 +1474,12 @@ void YProperty::discard() {
     }
 }
 
+Atom YProperty::fRequest;
+
 const YProperty& YProperty::update() {
     discard();
     int fmt = 0;
+    fRequest = fProp;
     if (XGetWindowProperty(xapp->display(), fWind, fProp, 0L, fLimit, fDelete,
                            fKind, &fType, &fmt, &fSize, &fMore, &fData) ==
         Success && fData && fSize && fmt == fBits && (fKind == fType || !fKind))
